from typing import Final

SIGNATURE_HMAC_SHA1: Final = "HMAC-SHA1"
SIGNATURE_RSA_SHA1: Final = "RSA-SHA1"
SIGNATURE_PLAINTEXT: Final = "PLAINTEXT"
SIGNATURE_TYPE_HEADER: Final = "HEADER"
SIGNATURE_TYPE_QUERY: Final = "QUERY"
SIGNATURE_TYPE_BODY: Final = "BODY"

def construct_base_string(method, uri, params, host=None) -> str: ...
def normalize_base_string_uri(uri, host=None): ...
def normalize_parameters(params) -> str: ...
def generate_signature_base_string(request) -> str: ...
def hmac_sha1_signature(base_string, client_secret, token_secret) -> str: ...
def rsa_sha1_signature(base_string, rsa_private_key) -> str: ...
def plaintext_signature(client_secret, token_secret) -> str: ...
def sign_hmac_sha1(client, request) -> str: ...
def sign_rsa_sha1(client, request) -> str: ...
def sign_plaintext(client, request) -> str: ...
def verify_hmac_sha1(request) -> bool: ...
def verify_rsa_sha1(request) -> bool: ...
def verify_plaintext(request) -> bool: ...



import abc
from _typeshed import Incomplete
from typing import type_check_only

from kafka.protocol.api import Request, Response

class ProduceResponse_v0(Response):
    API_KEY: int
    API_VERSION: int
    SCHEMA: Incomplete

class ProduceResponse_v1(Response):
    API_KEY: int
    API_VERSION: int
    SCHEMA: Incomplete

class ProduceResponse_v2(Response):
    API_KEY: int
    API_VERSION: int
    SCHEMA: Incomplete

class ProduceResponse_v3(Response):
    API_KEY: int
    API_VERSION: int
    SCHEMA: Incomplete

class ProduceResponse_v4(Response):
    API_KEY: int
    API_VERSION: int
    SCHEMA: Incomplete

class ProduceResponse_v5(Response):
    API_KEY: int
    API_VERSION: int
    SCHEMA: Incomplete

class ProduceResponse_v6(Response):
    API_KEY: int
    API_VERSION: int
    SCHEMA: Incomplete

class ProduceResponse_v7(Response):
    API_KEY: int
    API_VERSION: int
    SCHEMA: Incomplete

class ProduceResponse_v8(Response):
    API_KEY: int
    API_VERSION: int
    SCHEMA: Incomplete

@type_check_only
class _ProduceRequest(Request, metaclass=abc.ABCMeta):
    API_KEY: int
    def expect_response(self): ...

class ProduceRequest_v0(_ProduceRequest):
    API_VERSION: int
    RESPONSE_TYPE = ProduceResponse_v0
    SCHEMA: Incomplete

class ProduceRequest_v1(_ProduceRequest):
    API_VERSION: int
    RESPONSE_TYPE = ProduceResponse_v1
    SCHEMA: Incomplete

class ProduceRequest_v2(_ProduceRequest):
    API_VERSION: int
    RESPONSE_TYPE = ProduceResponse_v2
    SCHEMA: Incomplete

class ProduceRequest_v3(_ProduceRequest):
    API_VERSION: int
    RESPONSE_TYPE = ProduceResponse_v3
    SCHEMA: Incomplete

class ProduceRequest_v4(_ProduceRequest):
    API_VERSION: int
    RESPONSE_TYPE = ProduceResponse_v4
    SCHEMA: Incomplete

class ProduceRequest_v5(_ProduceRequest):
    API_VERSION: int
    RESPONSE_TYPE = ProduceResponse_v5
    SCHEMA: Incomplete

class ProduceRequest_v6(_ProduceRequest):
    API_VERSION: int
    RESPONSE_TYPE = ProduceResponse_v6
    SCHEMA: Incomplete

class ProduceRequest_v7(_ProduceRequest):
    API_VERSION: int
    RESPONSE_TYPE = ProduceResponse_v7
    SCHEMA: Incomplete

class ProduceRequest_v8(_ProduceRequest):
    API_VERSION: int
    RESPONSE_TYPE = ProduceResponse_v8
    SCHEMA: Incomplete

ProduceRequest: list[type[_ProduceRequest]]
ProduceResponse: Incomplete



import io
import re
from enum import IntEnum
from typing import Final

from gunicorn.config import Config
from gunicorn.http.body import Body
from gunicorn.http.unreader import Unreader

from .._types import _AddressType

PP_V2_SIGNATURE: Final = b"\x0d\x0a\x0d\x0a\x00\x0d\x0a\x51\x55\x49\x54\x0a"

class PPCommand(IntEnum):
    LOCAL = 0x0
    PROXY = 0x1

class PPFamily(IntEnum):
    UNSPEC = 0x0
    INET = 0x1
    INET6 = 0x2
    UNIX = 0x3

class PPProtocol(IntEnum):
    UNSPEC = 0x0
    STREAM = 0x1
    DGRAM = 0x2

MAX_REQUEST_LINE: Final = 8190
MAX_HEADERS: Final = 32768
DEFAULT_MAX_HEADERFIELD_SIZE: Final = 8190
RFC9110_5_6_2_TOKEN_SPECIALS: Final = r"!#$%&'*+-.^_`|~"
TOKEN_RE: Final[re.Pattern[str]]
METHOD_BADCHAR_RE: Final[re.Pattern[str]]
VERSION_RE: Final[re.Pattern[str]]
RFC9110_5_5_INVALID_AND_DANGEROUS: Final[re.Pattern[str]]
RFC9110_6_5_1_FORBIDDEN_TRAILER: Final[frozenset[str]]

class Message:
    cfg: Config
    unreader: Unreader
    peer_addr: _AddressType
    remote_addr: _AddressType
    version: tuple[int, int] | None
    headers: list[tuple[str, str]]
    trailers: list[tuple[str, str]]
    body: Body | None
    scheme: str
    must_close: bool
    limit_request_fields: int
    limit_request_field_size: int
    max_buffer_headers: int

    def __init__(self, cfg: Config, unreader: Unreader, peer_addr: _AddressType) -> None: ...
    def force_close(self) -> None: ...
    def parse(self, unreader: Unreader) -> bytes: ...
    def parse_headers(self, data: bytes, from_trailer: bool = False) -> list[tuple[str, str]]: ...
    def set_body_reader(self) -> None: ...
    def should_close(self) -> bool: ...

class Request(Message):
    method: str | None
    uri: str | None
    path: str | None
    query: str | None
    fragment: str | None
    limit_request_line: int
    req_number: int
    proxy_protocol_info: dict[str, str | int | None] | None  # TODO: Use TypedDict

    def __init__(self, cfg: Config, unreader: Unreader, peer_addr: _AddressType, req_number: int = 1) -> None: ...
    def get_data(self, unreader: Unreader, buf: io.BytesIO, stop: bool = False) -> None: ...
    def parse(self, unreader: Unreader) -> bytes: ...
    def read_into(self, unreader: Unreader, buf: bytearray, stop: bool | None = False) -> None: ...
    def read_line(self, unreader: Unreader, buf: bytearray, limit: int = 0) -> tuple[bytes, bytearray]: ...
    def read_bytes(self, unreader: Unreader, buf: bytearray, count: int) -> tuple[bytes, bytearray]: ...
    def proxy_protocol_access_check(self) -> None: ...
    def parse_request_line(self, line_bytes: bytes) -> None: ...
    def set_body_reader(self) -> None: ...



import sys
from _typeshed import structseq
from typing import Any, Final, final

if sys.platform != "win32":
    @final
    class struct_passwd(structseq[Any], tuple[str, str, int, int, str, str, str]):
        __match_args__: Final = ("pw_name", "pw_passwd", "pw_uid", "pw_gid", "pw_gecos", "pw_dir", "pw_shell")

        @property
        def pw_name(self) -> str: ...
        @property
        def pw_passwd(self) -> str: ...
        @property
        def pw_uid(self) -> int: ...
        @property
        def pw_gid(self) -> int: ...
        @property
        def pw_gecos(self) -> str: ...
        @property
        def pw_dir(self) -> str: ...
        @property
        def pw_shell(self) -> str: ...

    def getpwall() -> list[struct_passwd]: ...
    def getpwuid(uid: int, /) -> struct_passwd: ...
    def getpwnam(name: str, /) -> struct_passwd: ...



from _hashlib import HASH
from _typeshed import ReadableBuffer
from collections.abc import Callable
from typing import ClassVar, Final

from paramiko.message import Message
from paramiko.transport import Transport

c_MSG_KEXDH_GEX_REQUEST_OLD: Final[bytes]
c_MSG_KEXDH_GEX_GROUP: Final[bytes]
c_MSG_KEXDH_GEX_INIT: Final[bytes]
c_MSG_KEXDH_GEX_REPLY: Final[bytes]
c_MSG_KEXDH_GEX_REQUEST: Final[bytes]

class KexGexSHA256:
    name: ClassVar[str]
    min_bits: ClassVar[int]
    max_bits: ClassVar[int]
    preferred_bits: ClassVar[int]
    hash_algo: ClassVar[Callable[[ReadableBuffer], HASH]]

    transport: Transport
    p: int | None
    q: int | None
    g: int | None
    x: int | None
    e: int | None
    f: int | None
    old_style: bool

    def __init__(self, transport: Transport) -> None: ...
    def start_kex(self, _test_old_style: bool = False) -> None: ...
    def parse_next(self, ptype: int, m: Message) -> None: ...



import ssl
from _typeshed import Incomplete

log: Incomplete
DEFAULT_KAFKA_PORT: int
ssl_available: bool
SSLEOFError = ssl.SSLEOFError
SSLWantReadError = ssl.SSLWantReadError
SSLWantWriteError = ssl.SSLWantWriteError
SSLZeroReturnError = ssl.SSLZeroReturnError

AFI_NAMES: Incomplete

class ConnectionStates:
    DISCONNECTED: str
    CONNECTING: str
    HANDSHAKE: str
    CONNECTED: str
    AUTHENTICATING: str
    API_VERSIONS_SEND: str
    API_VERSIONS_RECV: str

class BrokerConnection:
    DEFAULT_CONFIG: Incomplete
    SECURITY_PROTOCOLS: Incomplete
    VERSION_CHECKS: Incomplete
    host: Incomplete
    port: Incomplete
    afi: Incomplete
    config: Incomplete
    node_id: Incomplete
    in_flight_requests: Incomplete
    state: Incomplete
    last_attempt: int
    def __init__(self, host, port, afi, **configs) -> None: ...
    def connect_blocking(self, timeout=...): ...
    def connect(self): ...
    def blacked_out(self): ...
    def throttled(self): ...
    def throttle_delay(self): ...
    def connection_delay(self): ...
    def connected(self): ...
    def connecting(self): ...
    def initializing(self): ...
    def disconnected(self): ...
    def connect_failed(self): ...
    def __del__(self) -> None: ...
    def close(self, error=None) -> None: ...
    def send(self, request, blocking: bool = True, request_timeout_ms=None): ...
    def send_pending_requests(self): ...
    def send_pending_requests_v2(self): ...
    def can_send_more(self): ...
    def recv(self): ...
    def requests_timed_out(self): ...
    def timed_out_ifrs(self): ...
    def next_ifr_request_timeout_ms(self): ...
    def get_api_versions(self): ...
    def check_version(self, timeout: int = 2, **kwargs): ...

class BrokerConnectionMetrics:
    metrics: Incomplete
    bytes_sent: Incomplete
    bytes_received: Incomplete
    request_time: Incomplete
    throttle_time: Incomplete
    def __init__(self, metrics, metric_group_prefix, node_id) -> None: ...

def get_ip_port_afi(host_and_port_str): ...
def is_inet_4_or_6(gai): ...
def dns_lookup(host, port, afi=...): ...



import ctypes
from dataclasses import dataclass
from typing import Final, Literal

from .gpio_pin_data import ChannelInfo

GPIO_HIGH: Final = 1

GPIOHANDLE_REQUEST_INPUT: Final = 0x1
GPIOHANDLE_REQUEST_OUTPUT: Final = 0x2

GPIOEVENT_REQUEST_RISING_EDGE: Final = 0x1
GPIOEVENT_REQUEST_FALLING_EDGE: Final = 0x2
GPIOEVENT_REQUEST_BOTH_EDGES: Final = 0x3

GPIO_GET_CHIPINFO_IOCTL: Final = 0x8044B401
GPIO_GET_LINEINFO_IOCTL: Final = 0xC048B402
GPIO_GET_LINEHANDLE_IOCTL: Final = 0xC16CB403
GPIOHANDLE_GET_LINE_VALUES_IOCTL: Final = 0xC040B408
GPIOHANDLE_SET_LINE_VALUES_IOCTL: Final = 0xC040B409
GPIO_GET_LINEEVENT_IOCTL: Final = 0xC030B404

class gpiochip_info(ctypes.Structure):
    name: str
    label: str
    lines: int

class gpiohandle_request(ctypes.Structure):
    lineoffsets: list[int]
    flags: int
    default_values: list[int]
    consumer_label: str
    lines: int
    fd: int

class gpiohandle_data(ctypes.Structure):
    values: list[int]

class gpioline_info(ctypes.Structure):
    line_offset: int
    flags: int
    name: str
    consumer: str

class gpioline_info_changed(ctypes.Structure):
    line_info: gpioline_info
    timestamp: int
    event_type: int
    padding: list[int]

class gpioevent_request(ctypes.Structure):
    lineoffset: int
    handleflags: int
    eventflags: int
    consumer_label: str
    fd: int

class gpioevent_data(ctypes.Structure):
    timestamp: int
    id: int

class GPIOError(IOError): ...

def chip_open(gpio_chip: str) -> int: ...
def chip_check_info(label: str, gpio_device: str) -> int | None: ...
def chip_open_by_label(label: str) -> int: ...
def close_chip(chip_fd: int) -> None: ...
def open_line(ch_info: ChannelInfo, request: int) -> None: ...
def close_line(line_handle: int) -> None: ...
def request_handle(line_offset: int, direction: Literal[0, 1], initial: Literal[0, 1], consumer: str) -> gpiohandle_request: ...
def request_event(line_offset: int, edge: int, consumer: str) -> gpioevent_request: ...
def get_value(line_handle: int) -> int: ...
def set_value(line_handle: int, value: int) -> None: ...

@dataclass
class PadCtlRegister:
    is_gpio: bool
    is_input: bool
    is_tristate: bool
    def __init__(self, value: int) -> None: ...
    @property
    def is_bidi(self) -> bool: ...

def check_pinmux(ch_info: ChannelInfo, direction: int) -> None: ...



from _typeshed import Incomplete, Unused
from collections.abc import Generator
from contextlib import contextmanager
from types import ModuleType
from typing import Literal, NoReturn

from win32.lib.pywintypes import IIDType
from win32com.client import dynamic

bForDemandDefault: int
clsidToTypelib: dict[str, tuple[str, int, int, int]]
versionRedirectMap: dict[tuple[str, int, int, int], ModuleType | None]
is_readonly: bool
is_zip: bool
demandGeneratedTypeLibraries: dict[tuple[str, int, int, int], Incomplete]

def __init__() -> None: ...

pickleVersion: int

@contextmanager
def ModuleMutex(module_name: str) -> Generator[None]: ...
def GetGeneratedFileName(clsid, lcid, major, minor) -> str: ...
def SplitGeneratedFileName(fname: str) -> tuple[str, ...]: ...
def GetGeneratePath() -> str: ...
def GetClassForProgID(progid: str) -> type | None: ...
def GetClassForCLSID(clsid) -> type | None: ...
def GetModuleForProgID(progid: str) -> ModuleType | None: ...
def GetModuleForCLSID(clsid) -> ModuleType | None: ...
def GetModuleForTypelib(typelibCLSID, lcid, major, minor) -> ModuleType: ...
def MakeModuleForTypelib(
    typelibCLSID,
    lcid,
    major,
    minor,
    progressInstance=None,
    bForDemand: bool | Literal[0, 1] = ...,
    bBuildHidden: bool | Literal[0, 1] = 1,
) -> ModuleType: ...
def MakeModuleForTypelibInterface(
    typelib_ob, progressInstance=None, bForDemand: bool | Literal[0, 1] = ..., bBuildHidden: bool | Literal[0, 1] = 1
) -> ModuleType | None: ...
def EnsureModuleForTypelibInterface(
    typelib_ob, progressInstance=None, bForDemand: bool | Literal[0, 1] = ..., bBuildHidden: bool | Literal[0, 1] = 1
) -> ModuleType | None: ...
def ForgetAboutTypelibInterface(typelib_ob) -> None: ...
def EnsureModule(
    typelibCLSID,
    lcid,
    major,
    minor,
    progressInstance=None,
    bValidateFile: bool | Literal[0, 1] = ...,
    bForDemand: bool | Literal[0, 1] = ...,
    bBuildHidden: bool | Literal[0, 1] = 1,
) -> ModuleType | None: ...
def EnsureDispatch(
    prog_id: str | dynamic.PyIDispatchType | IIDType | dynamic.PyIUnknownType, bForDemand: bool | Literal[0, 1] = 1
) -> dynamic.CDispatch: ...
def AddModuleToCache(typelibclsid, lcid, major, minor, verbose: Unused = 1, bFlushNow: bool | Literal[0, 1] = ...) -> None: ...
def GetGeneratedInfos() -> list[tuple[Incomplete, Incomplete, Incomplete, Incomplete]]: ...
def Rebuild(verbose: bool | Literal[0, 1] = 1) -> None: ...
def usage() -> NoReturn: ...



from collections.abc import Mapping
from enum import IntEnum

class ConfigResourceType(IntEnum):
    BROKER = 4
    TOPIC = 2

class ConfigResource:
    resource_type: ConfigResourceType
    name: str
    configs: Mapping[str, str] | None
    def __init__(self, resource_type: ConfigResourceType, name: str, configs: Mapping[str, str] | None = None) -> None: ...



import sys
from _typeshed import ReadableBuffer, SupportsNoArgReadline, SupportsRead, SupportsWrite

__all__ = [
    "encode",
    "decode",
    "encodebytes",
    "decodebytes",
    "b64encode",
    "b64decode",
    "b32encode",
    "b32decode",
    "b16encode",
    "b16decode",
    "b32hexencode",
    "b32hexdecode",
    "b85encode",
    "b85decode",
    "a85encode",
    "a85decode",
    "standard_b64encode",
    "standard_b64decode",
    "urlsafe_b64encode",
    "urlsafe_b64decode",
]

if sys.version_info >= (3, 13):
    __all__ += ["z85decode", "z85encode"]

if sys.version_info >= (3, 15):
    def b64encode(
        s: ReadableBuffer, altchars: ReadableBuffer | None = None, *, padded: bool = True, wrapcol: int = 0
    ) -> bytes: ...
    def b64decode(
        s: str | ReadableBuffer,
        altchars: str | ReadableBuffer | None = None,
        validate: bool = ...,
        *,
        padded: bool = True,
        ignorechars: ReadableBuffer = ...,
        canonical: bool = False,
    ) -> bytes: ...

else:
    def b64encode(s: ReadableBuffer, altchars: ReadableBuffer | None = None) -> bytes: ...
    def b64decode(s: str | ReadableBuffer, altchars: str | ReadableBuffer | None = None, validate: bool = False) -> bytes: ...

def standard_b64encode(s: ReadableBuffer) -> bytes: ...
def standard_b64decode(s: str | ReadableBuffer) -> bytes: ...

if sys.version_info >= (3, 15):
    def urlsafe_b64encode(s: ReadableBuffer, *, padded: bool = True) -> bytes: ...
    def urlsafe_b64decode(s: str | ReadableBuffer, *, padded: bool = False) -> bytes: ...
    def b32encode(s: ReadableBuffer, *, padded: bool = True, wrapcol: int = 0) -> bytes: ...
    def b32decode(
        s: str | ReadableBuffer,
        casefold: bool = False,
        map01: str | ReadableBuffer | None = None,
        *,
        padded: bool = True,
        ignorechars: ReadableBuffer = b"",
        canonical: bool = False,
    ) -> bytes: ...
    def b16encode(s: ReadableBuffer, *, wrapcol: int = 0) -> bytes: ...
    def b16decode(s: str | ReadableBuffer, casefold: bool = False, *, ignorechars: ReadableBuffer = b"") -> bytes: ...

else:
    def urlsafe_b64encode(s: ReadableBuffer) -> bytes: ...
    def urlsafe_b64decode(s: str | ReadableBuffer) -> bytes: ...
    def b32encode(s: ReadableBuffer) -> bytes: ...
    def b32decode(s: str | ReadableBuffer, casefold: bool = False, map01: str | ReadableBuffer | None = None) -> bytes: ...
    def b16encode(s: ReadableBuffer) -> bytes: ...
    def b16decode(s: str | ReadableBuffer, casefold: bool = False) -> bytes: ...

if sys.version_info >= (3, 15):
    def b32hexencode(s: ReadableBuffer, *, padded: bool = True, wrapcol: int = 0) -> bytes: ...
    def b32hexdecode(
        s: str | ReadableBuffer,
        casefold: bool = False,
        *,
        padded: bool = True,
        ignorechars: ReadableBuffer = b"",
        canonical: bool = False,
    ) -> bytes: ...

else:
    def b32hexencode(s: ReadableBuffer) -> bytes: ...
    def b32hexdecode(s: str | ReadableBuffer, casefold: bool = False) -> bytes: ...

def a85encode(
    b: ReadableBuffer, *, foldspaces: bool = False, wrapcol: int = 0, pad: bool = False, adobe: bool = False
) -> bytes: ...

if sys.version_info >= (3, 15):
    def a85decode(
        b: str | ReadableBuffer,
        *,
        foldspaces: bool = False,
        adobe: bool = False,
        ignorechars: bytearray | bytes = b" \t\n\r\x0b",
        canonical: bool = False,
    ) -> bytes: ...
    def b85encode(b: ReadableBuffer, pad: bool = False, *, wrapcol: int = 0) -> bytes: ...
    def b85decode(b: str | ReadableBuffer, *, ignorechars: ReadableBuffer = b"", canonical: bool = False) -> bytes: ...

else:
    def a85decode(
        b: str | ReadableBuffer, *, foldspaces: bool = False, adobe: bool = False, ignorechars: bytearray | bytes = b" \t\n\r\x0b"
    ) -> bytes: ...
    def b85encode(b: ReadableBuffer, pad: bool = False) -> bytes: ...
    def b85decode(b: str | ReadableBuffer) -> bytes: ...

def decode(input: SupportsNoArgReadline[bytes], output: SupportsWrite[bytes]) -> None: ...
def encode(input: SupportsRead[bytes], output: SupportsWrite[bytes]) -> None: ...
def encodebytes(s: ReadableBuffer) -> bytes: ...
def decodebytes(s: ReadableBuffer) -> bytes: ...

if sys.version_info >= (3, 13):
    if sys.version_info >= (3, 15):
        def z85encode(s: ReadableBuffer, pad: bool = False, *, wrapcol: int = 0) -> bytes: ...
        def z85decode(s: str | ReadableBuffer, *, ignorechars: ReadableBuffer = b"", canonical: bool = False) -> bytes: ...
    else:
        def z85encode(s: ReadableBuffer) -> bytes: ...
        def z85decode(s: str | ReadableBuffer) -> bytes: ...



from _typeshed import Incomplete
from collections.abc import Iterator
from typing_extensions import Self

from reportlab.pdfbase.pdfdoc import PDFObject

class PDFPattern(PDFObject):
    __RefOnly__: int
    pattern: Incomplete
    arguments: dict[str, Incomplete]
    def __init__(self, pattern_sequence, **keywordargs) -> None: ...
    def __setitem__(self, item: str, value) -> None: ...
    def __getitem__(self, item: str): ...
    def eval(self, L) -> Iterator[bytes]: ...
    def format(self, document) -> bytes: ...
    def clone(self) -> Self: ...

class PDFPatternIf:
    cond: Incomplete
    thenPart: Incomplete
    elsePart: Incomplete
    def __init__(self, cond, thenPart=[], elsePart=[]) -> None: ...



from collections.abc import Collection, Generator, Iterable, Sequence
from typing import Any, TypeAlias, TypedDict, TypeVar, overload, type_check_only

from bs4 import BeautifulSoup
from webtest.response import TestResponse

_T = TypeVar("_T")

@type_check_only
class _Classes(TypedDict):
    submit: type[Submit]
    button: type[Submit]
    image: type[Submit]
    multiple_select: type[MultipleSelect]
    select: type[Select]
    hidden: type[Hidden]
    file: type[File]
    text: type[Text]
    search: type[Text]
    email: type[Email]
    password: type[Text]
    checkbox: type[Checkbox]
    textarea: type[Textarea]
    radio: type[Radio]

# NOTE: It seems unergonomic having to put isinstance checks everywhere
#       in your test code where you're accessing a form field, so we
#       return `Any` for now. What we would really like to use here is
#       `AnyOf`, but that doesn't exist yet.
_AnyField: TypeAlias = Any

class NoValue: ...

class Upload:
    filename: str
    content: bytes | None
    content_type: str | None
    def __init__(self, filename: str, content: bytes | None = None, content_type: str | None = None) -> None: ...
    def __iter__(self) -> Generator[str | bytes]: ...

class Field:
    classes: _Classes
    form: Form
    tag: str
    name: str
    pos: int
    id: str
    attrs: dict[str, str]
    def __init__(
        self, form: Form, tag: str, name: str, pos: int, value: str | None = None, id: str | None = None, **attrs: str
    ) -> None: ...
    def value__get(self) -> str: ...
    def value__set(self, value: str | None) -> None: ...

    @property
    def value(self) -> str: ...
    @value.setter
    def value(self, value: str | None) -> None: ...

    def force_value(self, value: str | None) -> None: ...

class Select(Field):
    options: list[tuple[str, bool, str]]
    optionPositions: list[int]
    selectedIndex: int | None

    # NOTE: Even though it's safe to pass any object into text, I don't
    #       think that follows the spirit of this argument and is more
    #       likely a consequence of reusing the same utility function
    #       in order to handle bytes for Py2 compat.
    @overload
    def select(self, value: None, text: str | bytes) -> None: ...
    @overload
    def select(self, value: None = None, *, text: str | bytes) -> None: ...
    @overload
    def select(self, value: object, text: None = None) -> None: ...

    def value__get(self) -> str: ...
    def value__set(self, value: object | None) -> None: ...

    @property
    def value(self) -> str: ...
    @value.setter
    def value(self, value: object | None) -> None: ...

class MultipleSelect(Field):
    options: list[tuple[str, bool, str]]
    selectedIndices: list[int]

    @overload
    def select_multiple(self, value: None, texts: Iterable[str | bytes]) -> None: ...
    @overload
    def select_multiple(self, value: None = None, *, texts: Iterable[str | bytes]) -> None: ...
    @overload
    def select_multiple(self, value: Collection[object], texts: None = None) -> None: ...

    def value__get(self) -> list[str] | None: ...  # type: ignore[override]
    def value__set(self, values: Collection[object] | None) -> None: ...

    @property  # type: ignore[override]
    def value(self) -> list[str] | None: ...
    @value.setter
    def value(self, value: Collection[object] | None) -> None: ...

    # NOTE: Since unlike setting the value normally this doesn't perform
    #       any kind of type conversion, we're better off only allowing
    #       what `value__get` is supposed to be able to return.
    def force_value(self, values: list[str] | None) -> None: ...  # type: ignore[override]

class Radio(Select): ...

class Checkbox(Field):
    def value__get(self) -> str | None: ...  # type: ignore[override]
    def value__set(self, value: object) -> None: ...

    @property  # type: ignore[override]
    def value(self) -> str | None: ...
    @value.setter
    def value(self, value: object) -> None: ...

    def checked__get(self) -> bool: ...
    def checked__set(self, value: object) -> None: ...

    @property
    def checked(self) -> bool: ...
    @checked.setter
    def checked(self, value: object) -> None: ...

class Text(Field): ...
class Email(Field): ...
class File(Field): ...
class Textarea(Text): ...
class Hidden(Text): ...

class Submit(Field):
    def value__get(self) -> None: ...  # type: ignore[override]
    @property  # type: ignore[misc]
    def value(self) -> None: ...  # type: ignore[override]
    def value_if_submitted(self) -> str: ...

class Form:
    FieldClass: type[Field]
    response: TestResponse
    text: str
    html: BeautifulSoup
    action: str
    method: str
    id: str | None
    enctype: str
    field_order: list[tuple[str, Field]]
    fields: dict[str, list[Field]]
    def __init__(self, response: TestResponse, text: str, parser_features: Sequence[str] | str = "html.parser") -> None: ...
    # NOTE: Technically it is only safe to pass `str | None` for most fields
    #       but this method is not really usable if we don't lift this
    #       restriction, we just have to assume people know what they
    #       are doing
    def __setitem__(self, name: str, value: Any | None) -> None: ...
    def set(self, name: str, value: Any | None, index: int | None = None) -> None: ...
    def __getitem__(self, name: str) -> _AnyField: ...

    @overload
    def get(self, name: str, index: int | None = None) -> _AnyField: ...
    @overload
    def get(self, name: str, index: int | None, default: _T) -> _AnyField | _T: ...
    @overload
    def get(self, name: str, index: int | None = None, *, default: _T) -> _AnyField | _T: ...

    @overload
    def select(self, name: str, value: None, text: str | bytes, index: int | None = None) -> None: ...
    @overload
    def select(self, name: str, value: None = None, *, text: str | bytes, index: int | None = None) -> None: ...
    @overload
    def select(self, name: str, value: object, text: None = None, index: int | None = None) -> None: ...

    @overload
    def select_multiple(self, name: str, value: None, texts: Iterable[str | bytes], index: int | None = None) -> None: ...
    @overload
    def select_multiple(
        self, name: str, value: None = None, *, texts: Iterable[str | bytes], index: int | None = None
    ) -> None: ...
    @overload
    def select_multiple(self, name: str, value: Iterable[object], texts: None = None, index: int | None = None) -> None: ...

    def submit(
        self, name: str | None = None, index: int | None = None, value: str | None = None, **args: Any
    ) -> TestResponse: ...
    def lint(self) -> None: ...
    def upload_fields(self) -> list[tuple[str, str] | tuple[str, str, bytes]]: ...
    def submit_fields(
        self, name: str | None = None, index: int | None = None, submit_value: str | None = None
    ) -> list[tuple[str, str]]: ...



from typing import Any, Final

from docutils.parsers.rst import Directive

MODULEDOC: Final[str]
LEXERDOC: Final[str]
FMTERDOC: Final[str]
FILTERDOC: Final[str]

class PygmentsDoc(Directive):
    filenames: set[str]
    def document_lexers_overview(self) -> str: ...
    def document_lexers(self) -> str: ...
    def document_formatters(self) -> str: ...
    def document_filters(self) -> str: ...

def setup(app: Any) -> None: ...  # Actual type of 'app' is sphinx.application.Sphinx



from _typeshed import Incomplete

from kafka.sasl.abc import SaslMechanism

log: Incomplete

class SaslMechanismAwsMskIam(SaslMechanism):
    host: Incomplete
    def __init__(self, **config) -> None: ...
    def auth_bytes(self): ...
    def receive(self, auth_bytes) -> None: ...
    def is_done(self): ...
    def is_authenticated(self): ...
    def auth_details(self): ...

class AwsMskIamClient:
    UNRESERVED_CHARS: Incomplete
    algorithm: str
    expires: str
    hashfunc: Incomplete
    headers: Incomplete
    version: str
    service: str
    action: Incomplete
    datestamp: Incomplete
    timestamp: Incomplete
    host: Incomplete
    access_key: Incomplete
    secret_key: Incomplete
    region: Incomplete
    token: Incomplete
    def __init__(self, host, access_key, secret_key, region, token=None) -> None: ...
    def first_message(self): ...



from typing import Any

from .base import DictType

class Healthcheck(DictType[Any]):
    def __init__(
        self,
        *,
        test: str | list[str] | None = ...,
        Test: str | list[str] | None = ...,
        interval: int | None = ...,
        Interval: int | None = ...,
        timeout: int | None = ...,
        Timeout: int | None = ...,
        retries: int | None = ...,
        Retries: int | None = ...,
        start_period: int | None = ...,
        StartPeriod: int | None = ...,
    ) -> None: ...

    @property
    def test(self) -> list[str] | None: ...
    @test.setter
    def test(self, value: str | list[str] | None) -> None: ...

    @property
    def interval(self) -> int | None: ...
    @interval.setter
    def interval(self, value: int | None) -> None: ...

    @property
    def timeout(self) -> int | None: ...
    @timeout.setter
    def timeout(self, value: int | None) -> None: ...

    @property
    def retries(self) -> int | None: ...
    @retries.setter
    def retries(self, value: int | None) -> None: ...

    @property
    def start_period(self) -> int | None: ...
    @start_period.setter
    def start_period(self, value: int | None) -> None: ...



from _typeshed import Incomplete
from collections.abc import Generator
from typing import NoReturn, TypeAlias

from authlib.oauth1 import ClientAuth
from authlib.oauth1.client import OAuth1Client as _OAuth1Client

_Response: TypeAlias = Incomplete  # actual type is httpx.Response
_Request: TypeAlias = Incomplete  # actual type is httpx.Request

# Inherits from httpx.Auth
class OAuth1Auth(ClientAuth):
    requires_request_body: bool
    def auth_flow(self, request: _Request) -> Generator[_Request, _Response]: ...

# Inherits from httpx.AsyncClient
class AsyncOAuth1Client(_OAuth1Client):
    auth_class = OAuth1Auth
    def __init__(
        self,
        client_id,
        client_secret=None,
        token=None,
        token_secret=None,
        redirect_uri=None,
        rsa_key=None,
        verifier=None,
        signature_method=...,
        signature_type=...,
        force_include_body=False,
        **kwargs,
    ) -> None: ...
    async def fetch_access_token(self, url, verifier=None, **kwargs): ...
    @staticmethod
    def handle_error(error_type: str | None, error_description: str | None) -> NoReturn: ...

# Inherits from httpx.Client
class OAuth1Client(_OAuth1Client):
    auth_class = OAuth1Auth
    def __init__(
        self,
        client_id,
        client_secret=None,
        token=None,
        token_secret=None,
        redirect_uri=None,
        rsa_key=None,
        verifier=None,
        signature_method=...,
        signature_type=...,
        force_include_body=False,
        **kwargs,
    ) -> None: ...
    @staticmethod
    def handle_error(error_type: str | None, error_description: str | None) -> NoReturn: ...



from collections.abc import Mapping, Sequence

class NewTopic:
    name: str
    num_partitions: int
    replication_factor: int
    replica_assignments: Mapping[int, Sequence[int]] | None
    topic_configs: Mapping[str, str] | None
    def __init__(
        self,
        name: str,
        num_partitions: int = -1,
        replication_factor: int = -1,
        replica_assignments: Mapping[int, Sequence[int]] | None = None,
        topic_configs: Mapping[str, str] | None = None,
    ) -> None: ...



from _typeshed import Incomplete
from typing import Final

REQUESTS_SESSION_KWARGS: Final = ["proxies", "hooks", "stream", "verify", "cert", "max_redirects", "trust_env"]

def update_session_configure(session, kwargs: dict[str, Incomplete]) -> None: ...



from _typeshed import Incomplete

from kafka.record.abc import ABCRecord, ABCRecordBatch, ABCRecordBatchBuilder

class LegacyRecordBase:
    HEADER_STRUCT_V0: Incomplete
    HEADER_STRUCT_V1: Incomplete
    LOG_OVERHEAD: Incomplete
    CRC_OFFSET: Incomplete
    MAGIC_OFFSET: Incomplete
    RECORD_OVERHEAD_V0: Incomplete
    RECORD_OVERHEAD_V1: Incomplete
    KEY_OFFSET_V0: Incomplete
    KEY_OFFSET_V1: Incomplete
    KEY_LENGTH: Incomplete
    VALUE_LENGTH: Incomplete
    CODEC_MASK: int
    CODEC_NONE: int
    CODEC_GZIP: int
    CODEC_SNAPPY: int
    CODEC_LZ4: int
    TIMESTAMP_TYPE_MASK: int
    LOG_APPEND_TIME: int
    CREATE_TIME: int
    NO_TIMESTAMP: int

class LegacyRecordBatch(ABCRecordBatch, LegacyRecordBase):
    def __init__(self, buffer, magic) -> None: ...
    @property
    def base_offset(self): ...
    @property
    def size_in_bytes(self): ...
    @property
    def timestamp_type(self): ...
    @property
    def compression_type(self): ...
    @property
    def magic(self): ...
    def validate_crc(self): ...
    def __iter__(self): ...

class LegacyRecord(ABCRecord):
    def __init__(self, magic, offset, timestamp, timestamp_type, key, value, crc, crc_bytes) -> None: ...
    @property
    def magic(self): ...
    @property
    def offset(self): ...
    @property
    def timestamp(self): ...
    @property
    def timestamp_type(self): ...
    @property
    def key(self): ...
    @property
    def value(self): ...
    @property
    def headers(self): ...
    @property
    def checksum(self): ...
    def validate_crc(self): ...
    @property
    def size_in_bytes(self): ...

class LegacyRecordBatchBuilder(ABCRecordBatchBuilder, LegacyRecordBase):
    def __init__(self, magic, compression_type, batch_size) -> None: ...
    def append(self, offset, timestamp, key, value, headers=None): ...
    def build(self): ...
    def size(self): ...
    def size_in_bytes(self, offset, timestamp, key, value, headers=None): ...
    @classmethod
    def record_size(cls, magic, key, value): ...
    @classmethod
    def record_overhead(cls, magic): ...
    @classmethod
    def estimate_size_in_bytes(cls, magic, compression_type, key, value): ...

class LegacyRecordMetadata:
    def __init__(self, offset, crc, size, timestamp) -> None: ...
    @property
    def offset(self): ...
    @property
    def crc(self): ...
    @property
    def size(self): ...
    @property
    def timestamp(self): ...



from _typeshed import ReadableBuffer, Unused
from collections.abc import Callable, Mapping
from types import GenericAlias
from typing import Any, AnyStr, Generic, Literal, TypeVar, final, overload
from typing_extensions import Self

from . import _regex
from ._regex_core import *

_T = TypeVar("_T")

__version__: str

# Sync with regex.__init__.__all__
__all__ = [
    "cache_all",
    "compile",
    "DEFAULT_VERSION",
    "escape",
    "findall",
    "finditer",
    "fullmatch",
    "match",
    "prefixmatch",
    "purge",
    "search",
    "split",
    "splititer",
    "sub",
    "subf",
    "subfn",
    "subn",
    "template",
    "Scanner",
    "A",
    "ASCII",
    "B",
    "BESTMATCH",
    "D",
    "DEBUG",
    "E",
    "ENHANCEMATCH",
    "S",
    "DOTALL",
    "F",
    "FULLCASE",
    "I",
    "IGNORECASE",
    "L",
    "LOCALE",
    "M",
    "MULTILINE",
    "P",
    "POSIX",
    "R",
    "REVERSE",
    "T",
    "TEMPLATE",
    "U",
    "UNICODE",
    "V0",
    "VERSION0",
    "V1",
    "VERSION1",
    "X",
    "VERBOSE",
    "W",
    "WORD",
    "error",
    "Regex",
    "__version__",
    "__doc__",
    "RegexFlag",
    "Pattern",
    "Match",
]

def compile(
    pattern: AnyStr | Pattern[AnyStr],
    flags: int = 0,
    ignore_unused: bool = False,
    cache_pattern: bool | None = None,
    **kwargs: Any,
) -> Pattern[AnyStr]: ...

@overload
def search(
    pattern: str | Pattern[str],
    string: str,
    flags: int = 0,
    pos: int | None = None,
    endpos: int | None = None,
    partial: bool = False,
    concurrent: bool | None = None,
    timeout: float | None = None,
    ignore_unused: bool = False,
    **kwargs: Any,
) -> Match[str] | None: ...
@overload
def search(
    pattern: bytes | Pattern[bytes],
    string: ReadableBuffer,
    flags: int = 0,
    pos: int | None = None,
    endpos: int | None = None,
    partial: bool = False,
    concurrent: bool | None = None,
    timeout: float | None = None,
    ignore_unused: bool = False,
    **kwargs: Any,
) -> Match[bytes] | None: ...

@overload
def match(
    pattern: str | Pattern[str],
    string: str,
    flags: int = 0,
    pos: int | None = None,
    endpos: int | None = None,
    partial: bool = False,
    concurrent: bool | None = None,
    timeout: float | None = None,
    ignore_unused: bool = False,
    **kwargs: Any,
) -> Match[str] | None: ...
@overload
def match(
    pattern: bytes | Pattern[bytes],
    string: ReadableBuffer,
    flags: int = 0,
    pos: int | None = None,
    endpos: int | None = None,
    partial: bool = False,
    concurrent: bool | None = None,
    timeout: float | None = None,
    ignore_unused: bool = False,
    **kwargs: Any,
) -> Match[bytes] | None: ...

prefixmatch = match

@overload
def fullmatch(
    pattern: str | Pattern[str],
    string: str,
    flags: int = 0,
    pos: int | None = None,
    endpos: int | None = None,
    partial: bool = False,
    concurrent: bool | None = None,
    timeout: float | None = None,
    ignore_unused: bool = False,
    **kwargs: Any,
) -> Match[str] | None: ...
@overload
def fullmatch(
    pattern: bytes | Pattern[bytes],
    string: ReadableBuffer,
    flags: int = 0,
    pos: int | None = None,
    endpos: int | None = None,
    partial: bool = False,
    concurrent: bool | None = None,
    timeout: float | None = None,
    ignore_unused: bool = False,
    **kwargs: Any,
) -> Match[bytes] | None: ...

@overload
def split(
    pattern: str | Pattern[str],
    string: str,
    maxsplit: int = 0,
    flags: int = 0,
    concurrent: bool | None = None,
    timeout: float | None = None,
    ignore_unused: bool = False,
    **kwargs: Any,
) -> list[str | Any]: ...
@overload
def split(
    pattern: bytes | Pattern[bytes],
    string: ReadableBuffer,
    maxsplit: int = 0,
    flags: int = 0,
    concurrent: bool | None = None,
    timeout: float | None = None,
    ignore_unused: bool = False,
    **kwargs: Any,
) -> list[bytes | Any]: ...

@overload
def splititer(
    pattern: str | Pattern[str],
    string: str,
    maxsplit: int = 0,
    flags: int = 0,
    concurrent: bool | None = None,
    timeout: float | None = None,
    ignore_unused: bool = False,
    **kwargs: Any,
) -> _regex.Splitter[str]: ...
@overload
def splititer(
    pattern: bytes | Pattern[bytes],
    string: ReadableBuffer,
    maxsplit: int = 0,
    flags: int = 0,
    concurrent: bool | None = None,
    timeout: float | None = None,
    ignore_unused: bool = False,
    **kwargs: Any,
) -> _regex.Splitter[bytes]: ...

@overload
def findall(
    pattern: str | Pattern[str],
    string: str,
    flags: int = 0,
    pos: int | None = None,
    endpos: int | None = None,
    overlapped: bool = False,
    concurrent: bool | None = None,
    timeout: float | None = None,
    ignore_unused: bool = False,
    **kwargs: Any,
) -> list[Any]: ...
@overload
def findall(
    pattern: bytes | Pattern[bytes],
    string: ReadableBuffer,
    flags: int = 0,
    pos: int | None = None,
    endpos: int | None = None,
    overlapped: bool = False,
    concurrent: bool | None = None,
    timeout: float | None = None,
    ignore_unused: bool = False,
    **kwargs: Any,
) -> list[Any]: ...

@overload
def finditer(
    pattern: str | Pattern[str],
    string: str,
    flags: int = 0,
    pos: int | None = None,
    endpos: int | None = None,
    overlapped: bool = False,
    partial: bool = False,
    concurrent: bool | None = None,
    timeout: float | None = None,
    ignore_unused: bool = False,
    **kwargs: Any,
) -> _regex.Scanner[str]: ...
@overload
def finditer(
    pattern: bytes | Pattern[bytes],
    string: ReadableBuffer,
    flags: int = 0,
    pos: int | None = None,
    endpos: int | None = None,
    overlapped: bool = False,
    partial: bool = False,
    concurrent: bool | None = None,
    timeout: float | None = None,
    ignore_unused: bool = False,
    **kwargs: Any,
) -> _regex.Scanner[bytes]: ...

@overload
def sub(
    pattern: str | Pattern[str],
    repl: str | Callable[[Match[str]], str],
    string: str,
    count: int = 0,
    flags: int = 0,
    pos: int | None = None,
    endpos: int | None = None,
    concurrent: bool | None = None,
    timeout: float | None = None,
    ignore_unused: bool = False,
    **kwargs: Any,
) -> str: ...
@overload
def sub(
    pattern: bytes | Pattern[bytes],
    repl: ReadableBuffer | Callable[[Match[bytes]], ReadableBuffer],
    string: ReadableBuffer,
    count: int = 0,
    flags: int = 0,
    pos: int | None = None,
    endpos: int | None = None,
    concurrent: bool | None = None,
    timeout: float | None = None,
    ignore_unused: bool = False,
    **kwargs: Any,
) -> bytes: ...

@overload
def subf(
    pattern: str | Pattern[str],
    format: str | Callable[[Match[str]], str],
    string: str,
    count: int = 0,
    flags: int = 0,
    pos: int | None = None,
    endpos: int | None = None,
    concurrent: bool | None = None,
    timeout: float | None = None,
    ignore_unused: bool = False,
    **kwargs: Any,
) -> str: ...
@overload
def subf(
    pattern: bytes | Pattern[bytes],
    format: ReadableBuffer | Callable[[Match[bytes]], ReadableBuffer],
    string: ReadableBuffer,
    count: int = 0,
    flags: int = 0,
    pos: int | None = None,
    endpos: int | None = None,
    concurrent: bool | None = None,
    timeout: float | None = None,
    ignore_unused: bool = False,
    **kwargs: Any,
) -> bytes: ...

@overload
def subn(
    pattern: str | Pattern[str],
    repl: str | Callable[[Match[str]], str],
    string: str,
    count: int = 0,
    flags: int = 0,
    pos: int | None = None,
    endpos: int | None = None,
    concurrent: bool | None = None,
    timeout: float | None = None,
    ignore_unused: bool = False,
    **kwargs: Any,
) -> tuple[str, int]: ...
@overload
def subn(
    pattern: bytes | Pattern[bytes],
    repl: ReadableBuffer | Callable[[Match[bytes]], ReadableBuffer],
    string: ReadableBuffer,
    count: int = 0,
    flags: int = 0,
    pos: int | None = None,
    endpos: int | None = None,
    concurrent: bool | None = None,
    timeout: float | None = None,
    ignore_unused: bool = False,
    **kwargs: Any,
) -> tuple[bytes, int]: ...

@overload
def subfn(
    pattern: str | Pattern[str],
    format: str | Callable[[Match[str]], str],
    string: str,
    count: int = 0,
    flags: int = 0,
    pos: int | None = None,
    endpos: int | None = None,
    concurrent: bool | None = None,
    timeout: float | None = None,
    ignore_unused: bool = False,
    **kwargs: Any,
) -> tuple[str, int]: ...
@overload
def subfn(
    pattern: bytes | Pattern[bytes],
    format: ReadableBuffer | Callable[[Match[bytes]], ReadableBuffer],
    string: ReadableBuffer,
    count: int = 0,
    flags: int = 0,
    pos: int | None = None,
    endpos: int | None = None,
    concurrent: bool | None = None,
    timeout: float | None = None,
    ignore_unused: bool = False,
    **kwargs: Any,
) -> tuple[bytes, int]: ...

def purge() -> None: ...

@overload
def cache_all(value: bool = True) -> None: ...
@overload
def cache_all(value: None) -> bool: ...

def escape(pattern: AnyStr, special_only: bool = True, literal_spaces: bool = False) -> AnyStr: ...

DEFAULT_VERSION = RegexFlag.VERSION0

def template(pattern: AnyStr | Pattern[AnyStr], flags: int = 0) -> Pattern[AnyStr]: ...

Regex = compile

@final
class Pattern(Generic[AnyStr]):
    @property
    def flags(self) -> int: ...
    @property
    def groupindex(self) -> Mapping[str, int]: ...
    @property
    def groups(self) -> int: ...
    @property
    def pattern(self) -> AnyStr: ...
    @property
    def named_lists(self) -> Mapping[str, frozenset[AnyStr]]: ...

    @overload
    def search(
        self: Pattern[str],
        string: str,
        pos: int | None = None,
        endpos: int | None = None,
        concurrent: bool | None = None,
        partial: bool = False,
        timeout: float | None = None,
    ) -> Match[str] | None: ...
    @overload
    def search(
        self: Pattern[bytes],
        string: ReadableBuffer,
        pos: int | None = None,
        endpos: int | None = None,
        concurrent: bool | None = None,
        partial: bool = False,
        timeout: float | None = None,
    ) -> Match[bytes] | None: ...

    @overload
    def match(
        self: Pattern[str],
        string: str,
        pos: int | None = None,
        endpos: int | None = None,
        concurrent: bool | None = None,
        partial: bool = False,
        timeout: float | None = None,
    ) -> Match[str] | None: ...
    @overload
    def match(
        self: Pattern[bytes],
        string: ReadableBuffer,
        pos: int | None = None,
        endpos: int | None = None,
        concurrent: bool | None = None,
        partial: bool = False,
        timeout: float | None = None,
    ) -> Match[bytes] | None: ...

    prefixmatch = match

    @overload
    def fullmatch(
        self: Pattern[str],
        string: str,
        pos: int | None = None,
        endpos: int | None = None,
        concurrent: bool | None = None,
        partial: bool = False,
        timeout: float | None = None,
    ) -> Match[str] | None: ...
    @overload
    def fullmatch(
        self: Pattern[bytes],
        string: ReadableBuffer,
        pos: int | None = None,
        endpos: int | None = None,
        concurrent: bool | None = None,
        partial: bool = False,
        timeout: float | None = None,
    ) -> Match[bytes] | None: ...

    @overload
    def split(
        self: Pattern[str], string: str, maxsplit: int = 0, concurrent: bool | None = None, timeout: float | None = None
    ) -> list[str | Any]: ...
    @overload
    def split(
        self: Pattern[bytes],
        string: ReadableBuffer,
        maxsplit: int = 0,
        concurrent: bool | None = None,
        timeout: float | None = None,
    ) -> list[bytes | Any]: ...

    @overload
    def splititer(
        self: Pattern[str], string: str, maxsplit: int = 0, concurrent: bool | None = None, timeout: float | None = None
    ) -> _regex.Splitter[str]: ...
    @overload
    def splititer(
        self: Pattern[bytes],
        string: ReadableBuffer,
        maxsplit: int = 0,
        concurrent: bool | None = None,
        timeout: float | None = None,
    ) -> _regex.Splitter[bytes]: ...

    @overload
    def findall(
        self: Pattern[str],
        string: str,
        pos: int | None = None,
        endpos: int | None = None,
        overlapped: bool = False,
        concurrent: bool | None = None,
        timeout: float | None = None,
    ) -> list[Any]: ...
    @overload
    def findall(
        self: Pattern[bytes],
        string: ReadableBuffer,
        pos: int | None = None,
        endpos: int | None = None,
        overlapped: bool = False,
        concurrent: bool | None = None,
        timeout: float | None = None,
    ) -> list[Any]: ...

    @overload
    def finditer(
        self: Pattern[str],
        string: str,
        pos: int | None = None,
        endpos: int | None = None,
        overlapped: bool = False,
        concurrent: bool | None = None,
        partial: bool = False,
        timeout: float | None = None,
    ) -> _regex.Scanner[str]: ...
    @overload
    def finditer(
        self: Pattern[bytes],
        string: ReadableBuffer,
        pos: int | None = None,
        endpos: int | None = None,
        overlapped: bool = False,
        concurrent: bool | None = None,
        partial: bool = False,
        timeout: float | None = None,
    ) -> _regex.Scanner[bytes]: ...

    @overload
    def sub(
        self: Pattern[str],
        repl: str | Callable[[Match[str]], str],
        string: str,
        count: int = 0,
        pos: int | None = None,
        endpos: int | None = None,
        concurrent: bool | None = None,
        timeout: float | None = None,
    ) -> str: ...
    @overload
    def sub(
        self: Pattern[bytes],
        repl: ReadableBuffer | Callable[[Match[bytes]], ReadableBuffer],
        string: ReadableBuffer,
        count: int = 0,
        pos: int | None = None,
        endpos: int | None = None,
        concurrent: bool | None = None,
        timeout: float | None = None,
    ) -> bytes: ...

    @overload
    def subf(
        self: Pattern[str],
        format: str | Callable[[Match[str]], str],
        string: str,
        count: int = 0,
        pos: int | None = None,
        endpos: int | None = None,
        concurrent: bool | None = None,
        timeout: float | None = None,
    ) -> str: ...
    @overload
    def subf(
        self: Pattern[bytes],
        format: ReadableBuffer | Callable[[Match[bytes]], ReadableBuffer],
        string: ReadableBuffer,
        count: int = 0,
        pos: int | None = None,
        endpos: int | None = None,
        concurrent: bool | None = None,
        timeout: float | None = None,
    ) -> bytes: ...

    @overload
    def subn(
        self: Pattern[str],
        repl: str | Callable[[Match[str]], str],
        string: str,
        count: int = 0,
        pos: int | None = None,
        endpos: int | None = None,
        concurrent: bool | None = None,
        timeout: float | None = None,
    ) -> tuple[str, int]: ...
    @overload
    def subn(
        self: Pattern[bytes],
        repl: ReadableBuffer | Callable[[Match[bytes]], ReadableBuffer],
        string: ReadableBuffer,
        count: int = 0,
        pos: int | None = None,
        endpos: int | None = None,
        concurrent: bool | None = None,
        timeout: float | None = None,
    ) -> tuple[bytes, int]: ...

    @overload
    def subfn(
        self: Pattern[str],
        format: str | Callable[[Match[str]], str],
        string: str,
        count: int = 0,
        pos: int | None = None,
        endpos: int | None = None,
        concurrent: bool | None = None,
        timeout: float | None = None,
    ) -> tuple[str, int]: ...
    @overload
    def subfn(
        self: Pattern[bytes],
        format: ReadableBuffer | Callable[[Match[bytes]], ReadableBuffer],
        string: ReadableBuffer,
        count: int = 0,
        pos: int | None = None,
        endpos: int | None = None,
        concurrent: bool | None = None,
        timeout: float | None = None,
    ) -> tuple[bytes, int]: ...

    @overload
    def scanner(
        self: Pattern[str],
        string: str,
        pos: int | None = None,
        endpos: int | None = None,
        overlapped: bool = False,
        concurrent: bool | None = None,
        partial: bool = False,
        timeout: float | None = None,
    ) -> _regex.Scanner[str]: ...
    @overload
    def scanner(
        self: Pattern[bytes],
        string: bytes,
        pos: int | None = None,
        endpos: int | None = None,
        overlapped: bool = False,
        concurrent: bool | None = None,
        partial: bool = False,
        timeout: float | None = None,
    ) -> _regex.Scanner[bytes]: ...

    def __copy__(self) -> Self: ...
    def __deepcopy__(self, memo: Unused, /) -> Self: ...
    def __class_getitem__(cls, item: Any, /) -> GenericAlias: ...

@final
class Match(Generic[AnyStr]):
    @property
    def pos(self) -> int: ...
    @property
    def endpos(self) -> int: ...
    @property
    def lastindex(self) -> int | None: ...
    @property
    def lastgroup(self) -> str | None: ...
    @property
    def string(self) -> AnyStr: ...
    @property
    def re(self) -> Pattern[AnyStr]: ...
    @property
    def partial(self) -> bool: ...
    @property
    def regs(self) -> tuple[tuple[int, int], ...]: ...
    @property
    def fuzzy_counts(self) -> tuple[int, int, int]: ...
    @property
    def fuzzy_changes(self) -> tuple[list[int], list[int], list[int]]: ...

    @overload
    def group(self, group: Literal[0] = 0, /) -> AnyStr: ...
    @overload
    def group(self, group: int | str = ..., /) -> AnyStr | Any: ...
    @overload
    def group(self, group1: int | str, group2: int | str, /, *groups: int | str) -> tuple[AnyStr | Any, ...]: ...

    @overload
    def groups(self, default: None = None) -> tuple[AnyStr | Any, ...]: ...
    @overload
    def groups(self, default: _T) -> tuple[AnyStr | _T, ...]: ...

    @overload
    def groupdict(self, default: None = None) -> dict[str, AnyStr | Any]: ...
    @overload
    def groupdict(self, default: _T) -> dict[str, AnyStr | _T]: ...

    @overload
    def span(self, group: int | str = ..., /) -> tuple[int, int]: ...
    @overload
    def span(self, group1: int | str, group2: int | str, /, *groups: int | str) -> tuple[tuple[int, int], ...]: ...

    @overload
    def spans(self, group: int | str = ..., /) -> list[tuple[int, int]]: ...
    @overload
    def spans(self, group1: int | str, group2: int | str, /, *groups: int | str) -> tuple[list[tuple[int, int]], ...]: ...

    @overload
    def start(self, group: int | str = ..., /) -> int: ...
    @overload
    def start(self, group1: int | str, group2: int | str, /, *groups: int | str) -> tuple[int, ...]: ...

    @overload
    def starts(self, group: int | str = ..., /) -> list[int]: ...
    @overload
    def starts(self, group1: int | str, group2: int | str, /, *groups: int | str) -> tuple[list[int], ...]: ...

    @overload
    def end(self, group: int | str = ..., /) -> int: ...
    @overload
    def end(self, group1: int | str, group2: int | str, /, *groups: int | str) -> tuple[int, ...]: ...

    @overload
    def ends(self, group: int | str = ..., /) -> list[int]: ...
    @overload
    def ends(self, group1: int | str, group2: int | str, /, *groups: int | str) -> tuple[list[int], ...]: ...

    def expand(self, template: AnyStr, /) -> AnyStr: ...
    def expandf(self, format: AnyStr, /) -> AnyStr: ...

    @overload
    def captures(self, group: int | str = ..., /) -> list[AnyStr]: ...
    @overload
    def captures(self, group1: int | str, group2: int | str, /, *groups: int | str) -> tuple[list[AnyStr], ...]: ...

    def capturesdict(self) -> dict[str, list[AnyStr]]: ...
    def detach_string(self) -> None: ...
    def allcaptures(self) -> tuple[list[AnyStr]]: ...
    def allspans(self) -> tuple[list[tuple[int, int]]]: ...

    @overload
    def __getitem__(self, key: Literal[0], /) -> AnyStr: ...
    @overload
    def __getitem__(self, key: int | str, /) -> AnyStr | Any: ...

    def __copy__(self) -> Self: ...
    def __deepcopy__(self, memo: Unused, /) -> Self: ...
    def __class_getitem__(cls, item: Any, /) -> GenericAlias: ...



from _typeshed import Incomplete

from kafka.protocol.struct import Struct
from kafka.protocol.types import AbstractType

class Message(Struct):
    SCHEMAS: Incomplete
    SCHEMA: Incomplete
    CODEC_MASK: int
    CODEC_GZIP: int
    CODEC_SNAPPY: int
    CODEC_LZ4: int
    CODEC_ZSTD: int
    TIMESTAMP_TYPE_MASK: int
    HEADER_SIZE: int
    timestamp: Incomplete
    crc: Incomplete
    magic: Incomplete
    attributes: Incomplete
    key: Incomplete
    value: Incomplete
    encode: Incomplete
    def __init__(self, value, key=None, magic: int = 0, attributes: int = 0, crc: int = 0, timestamp=None) -> None: ...
    @property
    def timestamp_type(self): ...
    @classmethod
    def decode(cls, data): ...
    def validate_crc(self): ...
    def is_compressed(self): ...
    def decompress(self): ...
    def __hash__(self): ...

class PartialMessage(bytes): ...

class MessageSet(AbstractType):
    ITEM: Incomplete
    HEADER_SIZE: int
    @classmethod
    def encode(cls, items, prepend_size: bool = True): ...
    @classmethod
    def decode(cls, data, bytes_to_read=None): ...
    @classmethod
    def repr(cls, messages): ...



import sys
from _typeshed import FileDescriptorLike

def cancel_dump_traceback_later() -> None: ...
def disable() -> None: ...

if sys.version_info >= (3, 15):
    def dump_traceback(
        file: FileDescriptorLike = sys.stderr, all_threads: bool = True, *, max_threads: int | None = None
    ) -> None: ...

else:
    def dump_traceback(file: FileDescriptorLike = sys.stderr, all_threads: bool = True) -> None: ...

if sys.version_info >= (3, 14):
    def dump_c_stack(file: FileDescriptorLike = sys.stderr) -> None: ...

if sys.version_info >= (3, 15):
    def dump_traceback_later(
        timeout: float,
        repeat: bool = False,
        file: FileDescriptorLike = sys.stderr,
        exit: bool = False,
        *,
        max_threads: int | None = None,
    ) -> None: ...

else:
    def dump_traceback_later(
        timeout: float, repeat: bool = False, file: FileDescriptorLike = sys.stderr, exit: bool = False
    ) -> None: ...

if sys.version_info >= (3, 15):
    def enable(
        file: FileDescriptorLike = sys.stderr, all_threads: bool = True, c_stack: bool = True, *, max_threads: int | None = None
    ) -> None: ...

elif sys.version_info >= (3, 14):
    def enable(file: FileDescriptorLike = sys.stderr, all_threads: bool = True, c_stack: bool = True) -> None: ...

else:
    def enable(file: FileDescriptorLike = sys.stderr, all_threads: bool = True) -> None: ...

def is_enabled() -> bool: ...

if sys.platform != "win32":
    if sys.version_info >= (3, 15):
        def register(
            signum: int,
            file: FileDescriptorLike = sys.stderr,
            all_threads: bool = True,
            chain: bool = False,
            *,
            max_threads: int | None = None,
        ) -> None: ...
    else:
        def register(
            signum: int, file: FileDescriptorLike = sys.stderr, all_threads: bool = True, chain: bool = False
        ) -> None: ...

    def unregister(signum: int, /) -> None: ...



import datetime
from collections.abc import Callable
from typing import Any, Literal, ParamSpec, TypeVar
from typing_extensions import Self

_P = ParamSpec("_P")
_R = TypeVar("_R")

class Settings:
    # Next attributes are optional and may be missing.
    # Please keep in sync with _Settings TypedDict
    DATE_ORDER: str
    PREFER_LOCALE_DATE_ORDER: bool
    TIMEZONE: str
    TO_TIMEZONE: str
    RETURN_AS_TIMEZONE_AWARE: bool
    PREFER_MONTH_OF_YEAR: Literal["current", "first", "last"]
    PREFER_DAY_OF_MONTH: Literal["current", "first", "last"]
    PREFER_DATES_FROM: Literal["current_period", "future", "past"]
    RELATIVE_BASE: datetime.datetime
    STRICT_PARSING: bool
    REQUIRE_PARTS: list[Literal["day", "month", "year"]]
    SKIP_TOKENS: list[str]
    NORMALIZE: bool
    RETURN_TIME_AS_PERIOD: bool
    RETURN_TIME_SPAN: bool
    DEFAULT_START_OF_WEEK: Literal["monday", "sunday"]
    DEFAULT_DAYS_IN_MONTH: int
    PARSERS: list[Literal["timestamp", "relative-time", "custom-formats", "absolute-time", "no-spaces-time"]]
    DEFAULT_LANGUAGES: list[str]
    LANGUAGE_DETECTION_CONFIDENCE_THRESHOLD: float
    CACHE_SIZE_LIMIT: int

    def __new__(cls, *args, **kwargs) -> Self: ...
    def __init__(self, settings: dict[str, Any] | None = None) -> None: ...
    @classmethod
    def get_key(cls, settings: dict[str, Any] | None = None) -> str: ...
    def replace(self, mod_settings: dict[str, Any] | None = None, **kwds) -> Self: ...

settings: Settings

def apply_settings(f: Callable[_P, _R]) -> Callable[_P, _R]: ...

class SettingValidationError(ValueError): ...

def check_settings(settings: Settings) -> None: ...



from _typeshed import Incomplete
from typing import NamedTuple

from kafka.protocol.api import Request, Response

class AbortedTransaction(NamedTuple):
    producer_id: Incomplete
    first_offset: Incomplete

class FetchResponse_v0(Response):
    API_KEY: int
    API_VERSION: int
    SCHEMA: Incomplete

class FetchResponse_v1(Response):
    API_KEY: int
    API_VERSION: int
    SCHEMA: Incomplete

class FetchResponse_v2(Response):
    API_KEY: int
    API_VERSION: int
    SCHEMA: Incomplete

class FetchResponse_v3(Response):
    API_KEY: int
    API_VERSION: int
    SCHEMA: Incomplete

class FetchResponse_v4(Response):
    API_KEY: int
    API_VERSION: int
    SCHEMA: Incomplete

class FetchResponse_v5(Response):
    API_KEY: int
    API_VERSION: int
    SCHEMA: Incomplete

class FetchResponse_v6(Response):
    API_KEY: int
    API_VERSION: int
    SCHEMA: Incomplete

class FetchResponse_v7(Response):
    API_KEY: int
    API_VERSION: int
    SCHEMA: Incomplete

class FetchResponse_v8(Response):
    API_KEY: int
    API_VERSION: int
    SCHEMA: Incomplete

class FetchResponse_v9(Response):
    API_KEY: int
    API_VERSION: int
    SCHEMA: Incomplete

class FetchResponse_v10(Response):
    API_KEY: int
    API_VERSION: int
    SCHEMA: Incomplete

class FetchResponse_v11(Response):
    API_KEY: int
    API_VERSION: int
    SCHEMA: Incomplete

class FetchRequest_v0(Request):
    API_KEY: int
    API_VERSION: int
    RESPONSE_TYPE = FetchResponse_v0
    SCHEMA: Incomplete

class FetchRequest_v1(Request):
    API_KEY: int
    API_VERSION: int
    RESPONSE_TYPE = FetchResponse_v1
    SCHEMA: Incomplete

class FetchRequest_v2(Request):
    API_KEY: int
    API_VERSION: int
    RESPONSE_TYPE = FetchResponse_v2
    SCHEMA: Incomplete

class FetchRequest_v3(Request):
    API_KEY: int
    API_VERSION: int
    RESPONSE_TYPE = FetchResponse_v3
    SCHEMA: Incomplete

class FetchRequest_v4(Request):
    API_KEY: int
    API_VERSION: int
    RESPONSE_TYPE = FetchResponse_v4
    SCHEMA: Incomplete

class FetchRequest_v5(Request):
    API_KEY: int
    API_VERSION: int
    RESPONSE_TYPE = FetchResponse_v5
    SCHEMA: Incomplete

class FetchRequest_v6(Request):
    API_KEY: int
    API_VERSION: int
    RESPONSE_TYPE = FetchResponse_v6
    SCHEMA: Incomplete

class FetchRequest_v7(Request):
    API_KEY: int
    API_VERSION: int
    RESPONSE_TYPE = FetchResponse_v7
    SCHEMA: Incomplete

class FetchRequest_v8(Request):
    API_KEY: int
    API_VERSION: int
    RESPONSE_TYPE = FetchResponse_v8
    SCHEMA: Incomplete

class FetchRequest_v9(Request):
    API_KEY: int
    API_VERSION: int
    RESPONSE_TYPE = FetchResponse_v9
    SCHEMA: Incomplete

class FetchRequest_v10(Request):
    API_KEY: int
    API_VERSION: int
    RESPONSE_TYPE = FetchResponse_v10
    SCHEMA: Incomplete

class FetchRequest_v11(Request):
    API_KEY: int
    API_VERSION: int
    RESPONSE_TYPE = FetchResponse_v11
    SCHEMA: Incomplete

FetchRequest: Incomplete
FetchResponse: Incomplete



from .authorization_server import BaseServer as BaseServer, CacheAuthorizationServer as CacheAuthorizationServer
from .resource_protector import ResourceProtector as ResourceProtector

__all__ = ["BaseServer", "CacheAuthorizationServer", "ResourceProtector"]



import abc

class ABCRecord(metaclass=abc.ABCMeta):
    @property
    @abc.abstractmethod
    def size_in_bytes(self): ...
    @property
    @abc.abstractmethod
    def offset(self): ...
    @property
    @abc.abstractmethod
    def timestamp(self): ...
    @property
    @abc.abstractmethod
    def timestamp_type(self): ...
    @property
    @abc.abstractmethod
    def key(self): ...
    @property
    @abc.abstractmethod
    def value(self): ...
    @property
    @abc.abstractmethod
    def checksum(self): ...
    @abc.abstractmethod
    def validate_crc(self): ...
    @property
    @abc.abstractmethod
    def headers(self): ...

class ABCRecordBatchBuilder(metaclass=abc.ABCMeta):
    @abc.abstractmethod
    def append(self, offset, timestamp, key, value, headers=None): ...
    @abc.abstractmethod
    def size_in_bytes(self, offset, timestamp, key, value, headers): ...
    @abc.abstractmethod
    def build(self): ...

class ABCRecordBatch(metaclass=abc.ABCMeta):
    @abc.abstractmethod
    def __iter__(self): ...
    @property
    @abc.abstractmethod
    def base_offset(self): ...
    @property
    @abc.abstractmethod
    def size_in_bytes(self): ...
    @property
    @abc.abstractmethod
    def magic(self): ...

class ABCRecords(metaclass=abc.ABCMeta):
    @abc.abstractmethod
    def __init__(self, buffer): ...
    @abc.abstractmethod
    def size_in_bytes(self): ...
    @abc.abstractmethod
    def next_batch(self): ...
    @abc.abstractmethod
    def has_next(self): ...



import abc
from _typeshed import Incomplete

from kafka.protocol.struct import Struct

class RequestHeader(Struct):
    SCHEMA: Incomplete
    def __init__(self, request, correlation_id: int = 0, client_id: str = "kafka-python") -> None: ...

class RequestHeaderV2(Struct):
    SCHEMA: Incomplete
    def __init__(self, request, correlation_id: int = 0, client_id: str = "kafka-python", tags=None) -> None: ...

class ResponseHeader(Struct):
    SCHEMA: Incomplete

class ResponseHeaderV2(Struct):
    SCHEMA: Incomplete

class Request(Struct, metaclass=abc.ABCMeta):
    FLEXIBLE_VERSION: bool
    API_KEY: Incomplete
    API_VERSION: Incomplete
    SCHEMA: Incomplete
    RESPONSE_TYPE: Incomplete
    def expect_response(self): ...
    def to_object(self): ...
    def build_header(self, correlation_id, client_id): ...

class Response(Struct, metaclass=abc.ABCMeta):
    FLEXIBLE_VERSION: bool
    API_KEY: Incomplete
    API_VERSION: Incomplete
    SCHEMA: Incomplete
    def to_object(self): ...
    @classmethod
    def parse_header(cls, read_buffer): ...



import hashlib
from _typeshed import Incomplete

from authlib.jose.rfc7515 import JWSAlgorithm
from cryptography.hazmat.primitives import hashes
from cryptography.hazmat.primitives.asymmetric.padding import PKCS1v15

from .ec_key import ECKey
from .oct_key import OctKey
from .rsa_key import RSAKey

class NoneAlgorithm(JWSAlgorithm):
    name: str
    description: str
    deprecated: bool
    def prepare_key(self, raw_data) -> None: ...
    def sign(self, msg, key) -> bytes: ...
    def verify(self, msg, sig, key) -> bool: ...

class HMACAlgorithm(JWSAlgorithm):
    SHA256 = hashlib.sha256
    SHA384 = hashlib.sha384
    SHA512 = hashlib.sha512
    name: str
    description: str
    hash_alg: Incomplete
    def __init__(self, sha_type: int | str) -> None: ...
    def prepare_key(self, raw_data) -> OctKey: ...
    def sign(self, msg, key) -> bytes: ...
    def verify(self, msg, sig, key) -> bool: ...

class RSAAlgorithm(JWSAlgorithm):
    SHA256 = hashes.SHA256
    SHA384 = hashes.SHA384
    SHA512 = hashes.SHA512
    name: str
    description: str
    hash_alg: Incomplete
    padding: PKCS1v15
    def __init__(self, sha_type: int | str) -> None: ...
    def prepare_key(self, raw_data) -> RSAKey: ...
    def sign(self, msg, key): ...
    def verify(self, msg, sig, key) -> bool: ...

class ECAlgorithm(JWSAlgorithm):
    SHA256 = hashes.SHA256
    SHA384 = hashes.SHA384
    SHA512 = hashes.SHA512
    name: str
    curve: Incomplete
    description: str
    hash_alg: Incomplete
    def __init__(self, name: str, curve, sha_type: int | str) -> None: ...
    def prepare_key(self, raw_data) -> ECKey: ...
    def sign(self, msg, key) -> bytes: ...
    def verify(self, msg, sig, key) -> bool: ...

class RSAPSSAlgorithm(JWSAlgorithm):
    SHA256 = hashes.SHA256
    SHA384 = hashes.SHA384
    SHA512 = hashes.SHA512
    name: str
    description: str
    hash_alg: Incomplete
    def __init__(self, sha_type: int | str) -> None: ...
    def prepare_key(self, raw_data) -> RSAKey: ...
    def sign(self, msg, key): ...
    def verify(self, msg, sig, key) -> bool: ...

JWS_ALGORITHMS: list[JWSAlgorithm]



from _typeshed import Incomplete

from kafka.metrics.measurable_stat import AbstractMeasurableStat
from kafka.metrics.stats.sampled_stat import AbstractSampledStat

class TimeUnit:
    NANOSECONDS: Incomplete
    MICROSECONDS: Incomplete
    MILLISECONDS: Incomplete
    SECONDS: Incomplete
    MINUTES: Incomplete
    HOURS: Incomplete
    DAYS: Incomplete
    @staticmethod
    def get_name(time_unit): ...

class Rate(AbstractMeasurableStat):
    def __init__(self, time_unit=3, sampled_stat=None) -> None: ...
    def unit_name(self): ...
    def record(self, config, value, time_ms) -> None: ...
    def measure(self, config, now): ...
    def window_size(self, config, now): ...
    def convert(self, time_ms): ...

class SampledTotal(AbstractSampledStat):
    def __init__(self, initial_value=None) -> None: ...
    def update(self, sample, config, value, time_ms) -> None: ...
    def combine(self, samples, config, now): ...



from _typeshed import Incomplete

from ..base_client import FrameworkIntegration

token_update: Incomplete

class FlaskIntegration(FrameworkIntegration):
    def update_token(self, token, refresh_token=None, access_token=None) -> None: ...
    @staticmethod
    def load_config(oauth, name, params) -> dict[Incomplete, Incomplete]: ...



import selectors
import ssl
from _typeshed import Incomplete
from collections.abc import Callable, Iterable, Iterator, Mapping, Sequence
from typing import Literal, TypeAlias, TypedDict, overload, type_check_only
from typing_extensions import Self, Unpack

from kafka.consumer.fetcher import ConsumerRecord
from kafka.consumer.subscription_state import ConsumerRebalanceListener
from kafka.future import Future
from kafka.serializer.abstract import Deserializer
from kafka.structs import OffsetAndMetadata, OffsetAndTimestamp, TopicPartition

_ApiVersion: TypeAlias = tuple[int, ...]
_BootstrapServers: TypeAlias = str | Sequence[str]
_CommitCallback: TypeAlias = Callable[[Mapping[TopicPartition, OffsetAndMetadata], object], object]
_ConsumerDeserializer: TypeAlias = Deserializer | Callable[[bytes | None], object]
_KafkaClientFactory: TypeAlias = Callable[..., object]
_SaslMechanism: TypeAlias = Literal["PLAIN", "GSSAPI", "OAUTHBEARER", "SCRAM-SHA-256", "SCRAM-SHA-512"]
_SecurityProtocol: TypeAlias = Literal["PLAINTEXT", "SSL", "SASL_PLAINTEXT", "SASL_SSL"]
_SocketOption: TypeAlias = tuple[int, int, int]

@type_check_only
class _KafkaConsumerConfig(TypedDict, total=False):
    bootstrap_servers: _BootstrapServers
    client_id: str
    group_id: str | None
    group_instance_id: str | None
    key_deserializer: _ConsumerDeserializer | None
    value_deserializer: _ConsumerDeserializer | None
    enable_incremental_fetch_sessions: bool
    fetch_max_wait_ms: int
    fetch_min_bytes: int
    fetch_max_bytes: int
    max_partition_fetch_bytes: int
    request_timeout_ms: int
    retry_backoff_ms: int
    reconnect_backoff_ms: int
    reconnect_backoff_max_ms: int
    max_in_flight_requests_per_connection: int
    auto_offset_reset: Literal["earliest", "latest", "smallest", "largest"]
    enable_auto_commit: bool
    auto_commit_interval_ms: int
    default_offset_commit_callback: _CommitCallback
    check_crcs: bool
    isolation_level: Literal["read_uncommitted", "read_committed"]
    allow_auto_create_topics: bool
    metadata_max_age_ms: int
    partition_assignment_strategy: Sequence[type[object]]
    max_poll_records: int
    max_poll_interval_ms: int
    session_timeout_ms: int
    heartbeat_interval_ms: int
    receive_buffer_bytes: int | None
    send_buffer_bytes: int | None
    socket_options: Sequence[_SocketOption]
    sock_chunk_bytes: int
    sock_chunk_buffer_count: int
    consumer_timeout_ms: int | float
    security_protocol: _SecurityProtocol
    ssl_context: ssl.SSLContext | None
    ssl_check_hostname: bool
    ssl_cafile: str | None
    ssl_certfile: str | None
    ssl_keyfile: str | None
    ssl_crlfile: str | None
    ssl_password: str | None
    ssl_ciphers: str | None
    api_version: _ApiVersion | None
    api_version_auto_timeout_ms: int
    connections_max_idle_ms: int
    metric_reporters: Sequence[type[object]]
    metrics_enabled: bool
    metrics_num_samples: int
    metrics_sample_window_ms: int
    metric_group_prefix: str
    selector: type[selectors.BaseSelector]
    exclude_internal_topics: bool
    sasl_mechanism: _SaslMechanism | None
    sasl_plain_username: str | None
    sasl_plain_password: str | None
    sasl_kerberos_name: object | None
    sasl_kerberos_service_name: str
    sasl_kerberos_domain_name: str | None
    sasl_oauth_token_provider: object | None
    socks5_proxy: str | None
    kafka_client: _KafkaClientFactory

log: Incomplete

class KafkaConsumer(Iterator[ConsumerRecord]):
    DEFAULT_CONFIG: Incomplete
    DEFAULT_SESSION_TIMEOUT_MS_0_9: int
    config: Incomplete
    def __init__(self, *topics: str, **configs: Unpack[_KafkaConsumerConfig]) -> None: ...
    def bootstrap_connected(self): ...
    def assign(self, partitions: Iterable[TopicPartition]) -> None: ...
    def assignment(self) -> set[TopicPartition]: ...
    def close(self, autocommit: bool = True, timeout_ms: int | None = None) -> None: ...
    def commit_async(
        self, offsets: Mapping[TopicPartition, OffsetAndMetadata] | None = None, callback: _CommitCallback | None = None
    ) -> Future: ...
    def commit(
        self, offsets: Mapping[TopicPartition, OffsetAndMetadata] | None = None, timeout_ms: int | None = None
    ) -> None: ...

    @overload
    def committed(
        self, partition: TopicPartition, metadata: Literal[False] = False, timeout_ms: int | None = None
    ) -> int | None: ...
    @overload
    def committed(
        self, partition: TopicPartition, metadata: Literal[True], timeout_ms: int | None = None
    ) -> OffsetAndMetadata | None: ...
    @overload
    def committed(
        self, partition: TopicPartition, metadata: bool, timeout_ms: int | None = None
    ) -> int | OffsetAndMetadata | None: ...

    def topics(self) -> set[str]: ...
    def partitions_for_topic(self, topic: str) -> set[int]: ...
    def poll(
        self, timeout_ms: int = 0, max_records: int | None = None, update_offsets: bool = True
    ) -> dict[TopicPartition, list[ConsumerRecord]]: ...
    def position(self, partition: TopicPartition, timeout_ms: int | None = None) -> int | None: ...
    def highwater(self, partition: TopicPartition) -> int | None: ...
    def pause(self, *partitions: TopicPartition) -> None: ...
    def paused(self) -> set[TopicPartition]: ...
    def resume(self, *partitions: TopicPartition) -> None: ...
    def seek(self, partition: TopicPartition, offset: int) -> None: ...
    def seek_to_beginning(self, *partitions: TopicPartition) -> None: ...
    def seek_to_end(self, *partitions: TopicPartition) -> None: ...
    def subscribe(
        self, topics: Iterable[str] = (), pattern: str | None = None, listener: ConsumerRebalanceListener | None = None
    ) -> None: ...
    def subscription(self) -> set[str]: ...
    def unsubscribe(self) -> None: ...
    def metrics(self, raw: bool = False) -> dict[str, dict[str, object]] | dict[object, object] | None: ...
    def offsets_for_times(self, timestamps: Mapping[TopicPartition, int]) -> dict[TopicPartition, OffsetAndTimestamp | None]: ...
    def beginning_offsets(self, partitions: Iterable[TopicPartition]) -> dict[TopicPartition, int]: ...
    def end_offsets(self, partitions: Iterable[TopicPartition]) -> dict[TopicPartition, int]: ...
    def __iter__(self) -> Self: ...
    def __next__(self) -> ConsumerRecord: ...



from types import TracebackType

import ibm_db

class Db2connect:
    def __init__(self, dsn: str, username: str, password: str) -> None: ...
    def __enter__(self) -> ibm_db.IBM_DBConnection: ...
    def __exit__(self, t: type[BaseException] | None, v: BaseException | None, tb: TracebackType | None) -> None: ...



from _typeshed import Incomplete

log: Incomplete

class ProxyConnectionStates:
    DISCONNECTED: str
    CONNECTING: str
    NEGOTIATE_PROPOSE: str
    NEGOTIATING: str
    AUTHENTICATING: str
    REQUEST_SUBMIT: str
    REQUESTING: str
    READ_ADDRESS: str
    COMPLETE: str

class Socks5Wrapper:
    def __init__(self, proxy_url, afi) -> None: ...
    @classmethod
    def is_inet_4_or_6(cls, gai): ...
    @classmethod
    def dns_lookup(cls, host, port, afi=...): ...
    @classmethod
    def use_remote_lookup(cls, proxy_url): ...
    def socket(self, family, sock_type): ...
    def connect_ex(self, addr): ...



import builtins
import codecs
import sys
from _typeshed import FileDescriptorOrPath, MaybeNone, ReadableBuffer, WriteableBuffer
from collections.abc import Callable, Iterable, Iterator
from io import BufferedIOBase, RawIOBase, TextIOBase, UnsupportedOperation as UnsupportedOperation
from os import _Opener
from types import TracebackType
from typing import IO, Any, BinaryIO, Final, Generic, Literal, Protocol, TextIO, TypeVar, overload, type_check_only
from typing_extensions import Self, disjoint_base

_S = TypeVar("_S", bound=str)

if sys.version_info >= (3, 14):
    DEFAULT_BUFFER_SIZE: Final = 131072
else:
    DEFAULT_BUFFER_SIZE: Final = 8192

open = builtins.open

def open_code(path: str) -> IO[bytes]: ...

BlockingIOError = builtins.BlockingIOError

if sys.version_info >= (3, 12):
    @disjoint_base
    class _IOBase:
        def __iter__(self) -> Iterator[bytes]: ...
        def __next__(self) -> bytes: ...
        def __enter__(self) -> Self: ...
        def __exit__(
            self, exc_type: type[BaseException] | None, exc_val: BaseException | None, exc_tb: TracebackType | None
        ) -> None: ...
        def close(self) -> None: ...
        def fileno(self) -> int: ...
        def flush(self) -> None: ...
        def isatty(self) -> bool: ...
        def readable(self) -> bool: ...
        read: Callable[..., Any]
        def readlines(self, hint: int = -1, /) -> list[bytes]: ...
        def seek(self, offset: int, whence: int = 0, /) -> int: ...
        def seekable(self) -> bool: ...
        def tell(self) -> int: ...
        def truncate(self, size: int | None = None, /) -> int: ...
        def writable(self) -> bool: ...
        write: Callable[..., Any]
        def writelines(self, lines: Iterable[ReadableBuffer], /) -> None: ...
        def readline(self, size: int | None = -1, /) -> bytes: ...
        def __del__(self) -> None: ...
        @property
        def closed(self) -> bool: ...
        def _checkClosed(self) -> None: ...  # undocumented

else:
    class _IOBase:
        def __iter__(self) -> Iterator[bytes]: ...
        def __next__(self) -> bytes: ...
        def __enter__(self) -> Self: ...
        def __exit__(
            self, exc_type: type[BaseException] | None, exc_val: BaseException | None, exc_tb: TracebackType | None
        ) -> None: ...
        def close(self) -> None: ...
        def fileno(self) -> int: ...
        def flush(self) -> None: ...
        def isatty(self) -> bool: ...
        def readable(self) -> bool: ...
        read: Callable[..., Any]
        def readlines(self, hint: int = -1, /) -> list[bytes]: ...
        def seek(self, offset: int, whence: int = 0, /) -> int: ...
        def seekable(self) -> bool: ...
        def tell(self) -> int: ...
        def truncate(self, size: int | None = None, /) -> int: ...
        def writable(self) -> bool: ...
        write: Callable[..., Any]
        def writelines(self, lines: Iterable[ReadableBuffer], /) -> None: ...
        def readline(self, size: int | None = -1, /) -> bytes: ...
        def __del__(self) -> None: ...
        @property
        def closed(self) -> bool: ...
        def _checkClosed(self) -> None: ...  # undocumented

class _RawIOBase(_IOBase):
    def readall(self) -> bytes: ...
    # The following methods can return None if the file is in non-blocking mode
    # and no data is available.
    def readinto(self, buffer: WriteableBuffer, /) -> int | MaybeNone: ...
    def write(self, b: ReadableBuffer, /) -> int | MaybeNone: ...
    def read(self, size: int = -1, /) -> bytes | MaybeNone: ...

class _BufferedIOBase(_IOBase):
    def detach(self) -> RawIOBase: ...
    def readinto(self, buffer: WriteableBuffer, /) -> int: ...
    def write(self, buffer: ReadableBuffer, /) -> int: ...
    def readinto1(self, buffer: WriteableBuffer, /) -> int: ...
    def read(self, size: int | None = -1, /) -> bytes: ...
    def read1(self, size: int = -1, /) -> bytes: ...

@disjoint_base
class FileIO(RawIOBase, _RawIOBase, BinaryIO):  # type: ignore[misc]  # incompatible definitions of writelines in the base classes
    mode: str
    # The type of "name" equals the argument passed in to the constructor,
    # but that can make FileIO incompatible with other I/O types that assume
    # "name" is a str. In the future, making FileIO generic might help.
    name: Any
    def __init__(
        self, file: FileDescriptorOrPath, mode: str = "r", closefd: bool = True, opener: _Opener | None = None
    ) -> None: ...
    @property
    def closefd(self) -> bool: ...
    def seek(self, pos: int, whence: int = 0, /) -> int: ...
    def read(self, size: int | None = -1, /) -> bytes | MaybeNone: ...

@disjoint_base
class BytesIO(BufferedIOBase, _BufferedIOBase, BinaryIO):  # type: ignore[misc]  # incompatible definitions of methods in the base classes
    def __init__(self, initial_bytes: ReadableBuffer = b"") -> None: ...
    # BytesIO does not contain a "name" field. This workaround is necessary
    # to allow BytesIO sub-classes to add this field, as it is defined
    # as a read-only property on IO[].
    name: Any
    def getvalue(self) -> bytes: ...
    def getbuffer(self) -> memoryview: ...
    def read1(self, size: int | None = -1, /) -> bytes: ...
    def readlines(self, size: int | None = None, /) -> list[bytes]: ...
    def seek(self, pos: int, whence: int = 0, /) -> int: ...

@type_check_only
class _BufferedReaderStream(Protocol):
    def read(self, n: int = ..., /) -> bytes: ...
    # Optional: def readall(self) -> bytes: ...
    def readinto(self, b: memoryview, /) -> int | None: ...
    def seek(self, pos: int, whence: int, /) -> int: ...
    def tell(self) -> int: ...
    def truncate(self, size: int, /) -> int: ...
    def flush(self) -> object: ...
    def close(self) -> object: ...
    @property
    def closed(self) -> bool: ...
    def readable(self) -> bool: ...
    def seekable(self) -> bool: ...

    # The following methods just pass through to the underlying stream. Since
    # not all streams support them, they are marked as optional here, and will
    # raise an AttributeError if called on a stream that does not support them.

    # @property
    # def name(self) -> Any: ...  # Type is inconsistent between the various I/O types.
    # @property
    # def mode(self) -> str: ...
    # def fileno(self) -> int: ...
    # def isatty(self) -> bool: ...

_BufferedReaderStreamT = TypeVar("_BufferedReaderStreamT", bound=_BufferedReaderStream, default=_BufferedReaderStream)

@disjoint_base
class BufferedReader(BufferedIOBase, _BufferedIOBase, BinaryIO, Generic[_BufferedReaderStreamT]):  # type: ignore[misc]  # incompatible definitions of methods in the base classes
    raw: _BufferedReaderStreamT
    if sys.version_info >= (3, 14):
        def __init__(self, raw: _BufferedReaderStreamT, buffer_size: int = 131072) -> None: ...
    else:
        def __init__(self, raw: _BufferedReaderStreamT, buffer_size: int = 8192) -> None: ...

    def peek(self, size: int = 0, /) -> bytes: ...
    def seek(self, target: int, whence: int = 0, /) -> int: ...
    def truncate(self, pos: int | None = None, /) -> int: ...

@type_check_only
class _BufferedWriterStream(Protocol):
    def write(self, b: WriteableBuffer, /) -> int | None: ...
    def seek(self, pos: int, whence: int, /) -> int: ...
    def tell(self) -> int: ...
    def truncate(self, size: int, /) -> int: ...
    def flush(self) -> object: ...
    def close(self) -> object: ...
    @property
    def closed(self) -> bool: ...
    def writable(self) -> bool: ...
    def seekable(self) -> bool: ...

    # The following methods just pass through to the underlying stream. Since
    # not all streams support them, they are marked as optional here, and will
    # raise an AttributeError if called on a stream that does not support them.

    # @property
    # def name(self) -> Any: ...  # Type is inconsistent between the various I/O types.
    # @property
    # def mode(self) -> str: ...
    # def fileno(self) -> int: ...
    # def isatty(self) -> bool: ...

_BufferedWriterStreamT = TypeVar("_BufferedWriterStreamT", bound=_BufferedWriterStream, default=_BufferedWriterStream)

@disjoint_base
class BufferedWriter(BufferedIOBase, _BufferedIOBase, BinaryIO, Generic[_BufferedWriterStreamT]):  # type: ignore[misc]  # incompatible definitions of writelines in the base classes
    raw: _BufferedWriterStreamT
    if sys.version_info >= (3, 14):
        def __init__(self, raw: _BufferedWriterStreamT, buffer_size: int = 131072) -> None: ...
    else:
        def __init__(self, raw: _BufferedWriterStreamT, buffer_size: int = 8192) -> None: ...

    def write(self, buffer: ReadableBuffer, /) -> int: ...
    def seek(self, target: int, whence: int = 0, /) -> int: ...
    def truncate(self, pos: int | None = None, /) -> int: ...

@disjoint_base
class BufferedRandom(BufferedIOBase, _BufferedIOBase, BinaryIO):  # type: ignore[misc]  # incompatible definitions of methods in the base classes
    mode: str
    name: Any
    raw: RawIOBase
    if sys.version_info >= (3, 14):
        def __init__(self, raw: RawIOBase, buffer_size: int = 131072) -> None: ...
    else:
        def __init__(self, raw: RawIOBase, buffer_size: int = 8192) -> None: ...

    def seek(self, target: int, whence: int = 0, /) -> int: ...  # stubtest needs this
    def peek(self, size: int = 0, /) -> bytes: ...
    def truncate(self, pos: int | None = None, /) -> int: ...

@disjoint_base
class BufferedRWPair(BufferedIOBase, _BufferedIOBase, Generic[_BufferedReaderStreamT, _BufferedWriterStreamT]):
    if sys.version_info >= (3, 14):
        def __init__(
            self, reader: _BufferedReaderStreamT, writer: _BufferedWriterStreamT, buffer_size: int = 131072, /
        ) -> None: ...
    else:
        def __init__(
            self, reader: _BufferedReaderStreamT, writer: _BufferedWriterStreamT, buffer_size: int = 8192, /
        ) -> None: ...

    def peek(self, size: int = 0, /) -> bytes: ...

class _TextIOBase(_IOBase):
    encoding: str
    errors: str | None
    newlines: str | tuple[str, ...] | None
    def __iter__(self) -> Iterator[str]: ...  # type: ignore[override]
    def __next__(self) -> str: ...  # type: ignore[override]
    def detach(self) -> BinaryIO: ...
    def write(self, s: str, /) -> int: ...
    def writelines(self, lines: Iterable[str], /) -> None: ...  # type: ignore[override]
    def readline(self, size: int = -1, /) -> str: ...  # type: ignore[override]
    def readlines(self, hint: int = -1, /) -> list[str]: ...  # type: ignore[override]
    def read(self, size: int | None = -1, /) -> str: ...

@type_check_only
class _WrappedBuffer(Protocol):
    # "name" is wrapped by TextIOWrapper. Its type is inconsistent between
    # the various I/O types.
    @property
    def name(self) -> Any: ...
    @property
    def closed(self) -> bool: ...
    def read(self, size: int = ..., /) -> ReadableBuffer: ...
    # Optional: def read1(self, size: int, /) -> ReadableBuffer: ...
    def write(self, b: bytes, /) -> object: ...
    def flush(self) -> object: ...
    def close(self) -> object: ...
    def seekable(self) -> bool: ...
    def readable(self) -> bool: ...
    def writable(self) -> bool: ...
    def truncate(self, size: int, /) -> int: ...
    def fileno(self) -> int: ...
    def isatty(self) -> bool: ...
    # Optional: Only needs to be present if seekable() returns True.
    # def seek(self, offset: Literal[0], whence: Literal[2]) -> int: ...
    # def tell(self) -> int: ...

_BufferT_co = TypeVar("_BufferT_co", bound=_WrappedBuffer, default=_WrappedBuffer, covariant=True)

@disjoint_base
class TextIOWrapper(TextIOBase, _TextIOBase, TextIO, Generic[_BufferT_co]):  # type: ignore[misc]  # incompatible definitions of write in the base classes
    def __init__(
        self,
        buffer: _BufferT_co,
        encoding: str | None = None,
        errors: str | None = None,
        newline: str | None = None,
        line_buffering: bool = False,
        write_through: bool = False,
    ) -> None: ...
    # Equals the "buffer" argument passed in to the constructor.
    @property
    def buffer(self) -> _BufferT_co: ...  # type: ignore[override]
    @property
    def line_buffering(self) -> bool: ...
    @property
    def write_through(self) -> bool: ...
    def reconfigure(
        self,
        *,
        encoding: str | None = None,
        errors: str | None = None,
        newline: str | None = None,
        line_buffering: bool | None = None,
        write_through: bool | None = None,
    ) -> None: ...
    def readline(self, size: int = -1, /) -> str: ...  # type: ignore[override]
    # Equals the "buffer" argument passed in to the constructor.
    def detach(self) -> _BufferT_co: ...  # type: ignore[override]
    # TextIOWrapper's version of seek only supports a limited subset of
    # operations.
    def seek(self, cookie: int, whence: int = 0, /) -> int: ...
    def truncate(self, pos: int | None = None, /) -> int: ...

@disjoint_base
class StringIO(TextIOBase, _TextIOBase, TextIO):  # type: ignore[misc]  # incompatible definitions of write in the base classes
    def __init__(self, initial_value: str | None = "", newline: str | None = "\n") -> None: ...
    # StringIO does not contain a "name" field. This workaround is necessary
    # to allow StringIO sub-classes to add this field, as it is defined
    # as a read-only property on IO[].
    name: Any
    def getvalue(self) -> str: ...
    @property
    def line_buffering(self) -> bool: ...
    def seek(self, pos: int, whence: int = 0, /) -> int: ...
    def truncate(self, pos: int | None = None, /) -> int: ...

@disjoint_base
class IncrementalNewlineDecoder:
    def __init__(self, decoder: codecs.IncrementalDecoder | None, translate: bool, errors: str = "strict") -> None: ...
    def decode(self, input: ReadableBuffer | str, final: bool = False) -> str: ...
    @property
    def newlines(self) -> str | tuple[str, ...] | None: ...
    def getstate(self) -> tuple[bytes, int]: ...
    def reset(self) -> None: ...
    def setstate(self, state: tuple[bytes, int], /) -> None: ...

@overload
def text_encoding(encoding: None, stacklevel: int = 2, /) -> Literal["locale", "utf-8"]: ...
@overload
def text_encoding(encoding: _S, stacklevel: int = 2, /) -> _S: ...



from _typeshed import Incomplete

from kafka.coordinator.assignors.abstract import AbstractPartitionAssignor

log: Incomplete

class RangePartitionAssignor(AbstractPartitionAssignor):
    name: str
    version: int
    @classmethod
    def assign(cls, cluster, group_subscriptions): ...
    @classmethod
    def metadata(cls, topics): ...
    @classmethod
    def on_assignment(cls, assignment) -> None: ...



from _typeshed import Incomplete

from ..base_client import BaseOAuth, OAuthError as OAuthError
from .apps import StarletteOAuth1App as StarletteOAuth1App, StarletteOAuth2App as StarletteOAuth2App
from .integration import StarletteIntegration as StarletteIntegration

class OAuth(BaseOAuth):
    oauth1_client_cls = StarletteOAuth1App
    oauth2_client_cls = StarletteOAuth2App
    framework_integration_cls = StarletteIntegration
    config: Incomplete
    def __init__(self, config=None, cache=None, fetch_token=None, update_token=None) -> None: ...

__all__ = ["OAuth", "OAuthError", "StarletteIntegration", "StarletteOAuth1App", "StarletteOAuth2App"]



from _typeshed import SupportsRichComparison
from collections.abc import Callable, Iterable, Mapping
from typing import Any
from typing_extensions import Self

__tracebackhide__: bool

class ExtractingMixin:
    def extracting(
        self,
        *names: str,
        # The callable must accept the type of the items in the self.val collection.
        filter: str | Mapping[str, Any] | Callable[[Any], bool] = ...,
        sort: str | Iterable[str] | Callable[[Any], SupportsRichComparison] = ...,
    ) -> Self: ...



from _typeshed import Incomplete
from typing import NamedTuple

from kafka.protocol.api import Request, Response
from kafka.protocol.struct import Struct

DEFAULT_GENERATION_ID: int
UNKNOWN_MEMBER_ID: str

class GroupMember(NamedTuple):
    member_id: Incomplete = ...
    group_instance_id: Incomplete = ...
    metadata_bytes: Incomplete = ...

class JoinGroupResponse_v0(Response):
    API_KEY: int
    API_VERSION: int
    SCHEMA: Incomplete

class JoinGroupResponse_v1(Response):
    API_KEY: int
    API_VERSION: int
    SCHEMA: Incomplete

class JoinGroupResponse_v2(Response):
    API_KEY: int
    API_VERSION: int
    SCHEMA: Incomplete

class JoinGroupResponse_v3(Response):
    API_KEY: int
    API_VERSION: int
    SCHEMA: Incomplete

class JoinGroupResponse_v4(Response):
    API_KEY: int
    API_VERSION: int
    SCHEMA: Incomplete

class JoinGroupResponse_v5(Response):
    API_KEY: int
    API_VERSION: int
    SCHEMA: Incomplete

class JoinGroupRequest_v0(Request):
    API_KEY: int
    API_VERSION: int
    RESPONSE_TYPE = JoinGroupResponse_v0
    SCHEMA: Incomplete

class JoinGroupRequest_v1(Request):
    API_KEY: int
    API_VERSION: int
    RESPONSE_TYPE = JoinGroupResponse_v1
    SCHEMA: Incomplete

class JoinGroupRequest_v2(Request):
    API_KEY: int
    API_VERSION: int
    RESPONSE_TYPE = JoinGroupResponse_v2
    SCHEMA: Incomplete

class JoinGroupRequest_v3(Request):
    API_KEY: int
    API_VERSION: int
    RESPONSE_TYPE = JoinGroupResponse_v3
    SCHEMA: Incomplete

class JoinGroupRequest_v4(Request):
    API_KEY: int
    API_VERSION: int
    RESPONSE_TYPE = JoinGroupResponse_v4
    SCHEMA: Incomplete

class JoinGroupRequest_v5(Request):
    API_KEY: int
    API_VERSION: int
    RESPONSE_TYPE = JoinGroupResponse_v5
    SCHEMA: Incomplete

JoinGroupRequest: Incomplete
JoinGroupResponse: Incomplete

class ProtocolMetadata(Struct):
    SCHEMA: Incomplete

class SyncGroupResponse_v0(Response):
    API_KEY: int
    API_VERSION: int
    SCHEMA: Incomplete

class SyncGroupResponse_v1(Response):
    API_KEY: int
    API_VERSION: int
    SCHEMA: Incomplete

class SyncGroupResponse_v2(Response):
    API_KEY: int
    API_VERSION: int
    SCHEMA: Incomplete

class SyncGroupResponse_v3(Response):
    API_KEY: int
    API_VERSION: int
    SCHEMA: Incomplete

class SyncGroupRequest_v0(Request):
    API_KEY: int
    API_VERSION: int
    RESPONSE_TYPE = SyncGroupResponse_v0
    SCHEMA: Incomplete

class SyncGroupRequest_v1(Request):
    API_KEY: int
    API_VERSION: int
    RESPONSE_TYPE = SyncGroupResponse_v1
    SCHEMA: Incomplete

class SyncGroupRequest_v2(Request):
    API_KEY: int
    API_VERSION: int
    RESPONSE_TYPE = SyncGroupResponse_v2
    SCHEMA: Incomplete

class SyncGroupRequest_v3(Request):
    API_KEY: int
    API_VERSION: int
    RESPONSE_TYPE = SyncGroupResponse_v3
    SCHEMA: Incomplete

SyncGroupRequest: Incomplete
SyncGroupResponse: Incomplete

class MemberAssignment(Struct):
    SCHEMA: Incomplete

class HeartbeatResponse_v0(Response):
    API_KEY: int
    API_VERSION: int
    SCHEMA: Incomplete

class HeartbeatResponse_v1(Response):
    API_KEY: int
    API_VERSION: int
    SCHEMA: Incomplete

class HeartbeatResponse_v2(Response):
    API_KEY: int
    API_VERSION: int
    SCHEMA: Incomplete

class HeartbeatResponse_v3(Response):
    API_KEY: int
    API_VERSION: int
    SCHEMA: Incomplete

class HeartbeatRequest_v0(Request):
    API_KEY: int
    API_VERSION: int
    RESPONSE_TYPE = HeartbeatResponse_v0
    SCHEMA: Incomplete

class HeartbeatRequest_v1(Request):
    API_KEY: int
    API_VERSION: int
    RESPONSE_TYPE = HeartbeatResponse_v1
    SCHEMA: Incomplete

class HeartbeatRequest_v2(Request):
    API_KEY: int
    API_VERSION: int
    RESPONSE_TYPE = HeartbeatResponse_v2
    SCHEMA: Incomplete

class HeartbeatRequest_v3(Request):
    API_KEY: int
    API_VERSION: int
    RESPONSE_TYPE = HeartbeatResponse_v3
    SCHEMA: Incomplete

HeartbeatRequest: Incomplete
HeartbeatResponse: Incomplete

class LeaveGroupResponse_v0(Response):
    API_KEY: int
    API_VERSION: int
    SCHEMA: Incomplete

class LeaveGroupResponse_v1(Response):
    API_KEY: int
    API_VERSION: int
    SCHEMA: Incomplete

class LeaveGroupResponse_v2(Response):
    API_KEY: int
    API_VERSION: int
    SCHEMA: Incomplete

class LeaveGroupResponse_v3(Response):
    API_KEY: int
    API_VERSION: int
    SCHEMA: Incomplete

class LeaveGroupRequest_v0(Request):
    API_KEY: int
    API_VERSION: int
    RESPONSE_TYPE = LeaveGroupResponse_v0
    SCHEMA: Incomplete

class LeaveGroupRequest_v1(Request):
    API_KEY: int
    API_VERSION: int
    RESPONSE_TYPE = LeaveGroupResponse_v1
    SCHEMA: Incomplete

class LeaveGroupRequest_v2(Request):
    API_KEY: int
    API_VERSION: int
    RESPONSE_TYPE = LeaveGroupResponse_v2
    SCHEMA: Incomplete

class LeaveGroupRequest_v3(Request):
    API_KEY: int
    API_VERSION: int
    RESPONSE_TYPE = LeaveGroupResponse_v3
    SCHEMA: Incomplete

LeaveGroupRequest: Incomplete
LeaveGroupResponse: Incomplete



from authlib.oauth2.rfc6749 import TokenEndpoint

class IntrospectionEndpoint(TokenEndpoint):
    ENDPOINT_NAME: str
    def authenticate_token(self, request, client): ...
    def check_params(self, request, client) -> None: ...
    def create_endpoint_response(self, request): ...
    def create_introspection_payload(self, token): ...
    def check_permission(self, token, client, request): ...
    def query_token(self, token_string, token_type_hint): ...
    def introspect_token(self, token): ...



import sys
from _typeshed import StrPath
from collections.abc import Callable, Generator, Sequence
from contextlib import contextmanager
from tkinter import Canvas, Frame, Misc, PhotoImage, Scrollbar
from typing import Any, ClassVar, Literal, TypeAlias, TypedDict, overload, type_check_only
from typing_extensions import Self, deprecated, disjoint_base

__all__ = [
    "ScrolledCanvas",
    "TurtleScreen",
    "Screen",
    "RawTurtle",
    "Turtle",
    "RawPen",
    "Pen",
    "Shape",
    "Vec2D",
    "addshape",
    "bgcolor",
    "bgpic",
    "bye",
    "clearscreen",
    "colormode",
    "delay",
    "exitonclick",
    "getcanvas",
    "getshapes",
    "listen",
    "mainloop",
    "mode",
    "numinput",
    "onkey",
    "onkeypress",
    "onkeyrelease",
    "onscreenclick",
    "ontimer",
    "register_shape",
    "resetscreen",
    "screensize",
    "setup",
    "setworldcoordinates",
    "textinput",
    "title",
    "tracer",
    "turtles",
    "update",
    "window_height",
    "window_width",
    "back",
    "backward",
    "begin_fill",
    "begin_poly",
    "bk",
    "circle",
    "clear",
    "clearstamp",
    "clearstamps",
    "clone",
    "color",
    "degrees",
    "distance",
    "dot",
    "down",
    "end_fill",
    "end_poly",
    "fd",
    "fillcolor",
    "filling",
    "forward",
    "get_poly",
    "getpen",
    "getscreen",
    "get_shapepoly",
    "getturtle",
    "goto",
    "heading",
    "hideturtle",
    "home",
    "ht",
    "isdown",
    "isvisible",
    "left",
    "lt",
    "onclick",
    "ondrag",
    "onrelease",
    "pd",
    "pen",
    "pencolor",
    "pendown",
    "pensize",
    "penup",
    "pos",
    "position",
    "pu",
    "radians",
    "right",
    "reset",
    "resizemode",
    "rt",
    "seth",
    "setheading",
    "setpos",
    "setposition",
    "setundobuffer",
    "setx",
    "sety",
    "shape",
    "shapesize",
    "shapetransform",
    "shearfactor",
    "showturtle",
    "speed",
    "st",
    "stamp",
    "tilt",
    "tiltangle",
    "towards",
    "turtlesize",
    "undo",
    "undobufferentries",
    "up",
    "width",
    "write",
    "xcor",
    "ycor",
    "write_docstringdict",
    "done",
    "Terminator",
]

if sys.version_info >= (3, 14):
    __all__ += ["fill", "no_animation", "poly", "save"]

if sys.version_info >= (3, 12):
    __all__ += ["teleport"]

if sys.version_info < (3, 13):
    __all__ += ["settiltangle"]

# Note: '_Color' is the alias we use for arguments and _AnyColor is the
# alias we use for return types. Really, these two aliases should be the
# same, but as per the "no union returns" typeshed policy, we'll return
# Any instead.
_Color: TypeAlias = str | tuple[float, float, float]
_AnyColor: TypeAlias = Any

@type_check_only
class _PenState(TypedDict):
    shown: bool
    pendown: bool
    pencolor: _Color
    fillcolor: _Color
    pensize: int
    speed: int
    resizemode: Literal["auto", "user", "noresize"]
    stretchfactor: tuple[float, float]
    shearfactor: float
    outline: int
    tilt: float

_Speed: TypeAlias = str | float
_PolygonCoords: TypeAlias = Sequence[tuple[float, float]]

if sys.version_info >= (3, 12):
    class Vec2D(tuple[float, float]):
        def __new__(cls, x: float, y: float) -> Self: ...
        def __add__(self, other: tuple[float, float]) -> Vec2D: ...  # type: ignore[override]

        @overload  # type: ignore[override]
        def __mul__(self, other: Vec2D) -> float: ...
        @overload
        def __mul__(self, other: float) -> Vec2D: ...

        def __rmul__(self, other: float) -> Vec2D: ...  # type: ignore[override]
        def __sub__(self, other: tuple[float, float]) -> Vec2D: ...
        def __neg__(self) -> Vec2D: ...
        def __abs__(self) -> float: ...
        def rotate(self, angle: float) -> Vec2D: ...

else:
    @disjoint_base
    class Vec2D(tuple[float, float]):
        def __new__(cls, x: float, y: float) -> Self: ...
        def __add__(self, other: tuple[float, float]) -> Vec2D: ...  # type: ignore[override]

        @overload  # type: ignore[override]
        def __mul__(self, other: Vec2D) -> float: ...
        @overload
        def __mul__(self, other: float) -> Vec2D: ...

        def __rmul__(self, other: float) -> Vec2D: ...  # type: ignore[override]
        def __sub__(self, other: tuple[float, float]) -> Vec2D: ...
        def __neg__(self) -> Vec2D: ...
        def __abs__(self) -> float: ...
        def rotate(self, angle: float) -> Vec2D: ...

# Does not actually inherit from Canvas, but dynamically gets all methods of Canvas
class ScrolledCanvas(Canvas, Frame):  # type: ignore[misc]
    bg: str
    hscroll: Scrollbar
    vscroll: Scrollbar
    def __init__(
        self, master: Misc | None, width: int = 500, height: int = 350, canvwidth: int = 600, canvheight: int = 500
    ) -> None: ...
    canvwidth: int
    canvheight: int
    def reset(self, canvwidth: int | None = None, canvheight: int | None = None, bg: str | None = None) -> None: ...

class TurtleScreenBase:
    cv: Canvas
    canvwidth: int
    canvheight: int
    xscale: float
    yscale: float
    def __init__(self, cv: Canvas) -> None: ...
    def mainloop(self) -> None: ...
    def textinput(self, title: str, prompt: str) -> str | None: ...
    def numinput(
        self, title: str, prompt: str, default: float | None = None, minval: float | None = None, maxval: float | None = None
    ) -> float | None: ...

class Terminator(Exception): ...
class TurtleGraphicsError(Exception): ...

class Shape:
    def __init__(
        self, type_: Literal["polygon", "image", "compound"], data: _PolygonCoords | PhotoImage | None = None
    ) -> None: ...
    def addcomponent(self, poly: _PolygonCoords, fill: _Color, outline: _Color | None = None) -> None: ...

class TurtleScreen(TurtleScreenBase):
    def __init__(
        self, cv: Canvas, mode: Literal["standard", "logo", "world"] = "standard", colormode: float = 1.0, delay: int = 10
    ) -> None: ...
    def clear(self) -> None: ...

    @overload
    def mode(self, mode: None = None) -> str: ...
    @overload
    def mode(self, mode: Literal["standard", "logo", "world"]) -> None: ...

    def setworldcoordinates(self, llx: float, lly: float, urx: float, ury: float) -> None: ...
    def register_shape(self, name: str, shape: _PolygonCoords | Shape | None = None) -> None: ...

    @overload
    def colormode(self, cmode: None = None) -> float: ...
    @overload
    def colormode(self, cmode: float) -> None: ...

    def reset(self) -> None: ...
    def turtles(self) -> list[Turtle]: ...

    @overload
    def bgcolor(self) -> _AnyColor: ...
    @overload
    def bgcolor(self, color: _Color) -> None: ...
    @overload
    def bgcolor(self, r: float, g: float, b: float) -> None: ...

    @overload
    def tracer(self, n: None = None) -> int: ...
    @overload
    def tracer(self, n: int, delay: int | None = None) -> None: ...

    @overload
    def delay(self, delay: None = None) -> int: ...
    @overload
    def delay(self, delay: int) -> None: ...

    if sys.version_info >= (3, 14):
        @contextmanager
        def no_animation(self) -> Generator[None]: ...

    def update(self) -> None: ...
    def window_width(self) -> int: ...
    def window_height(self) -> int: ...
    def getcanvas(self) -> Canvas: ...
    def getshapes(self) -> list[str]: ...
    def onclick(self, fun: Callable[[float, float], object], btn: int = 1, add: bool | None = None) -> None: ...
    def onkey(self, fun: Callable[[], object], key: str) -> None: ...
    def listen(self, xdummy: float | None = None, ydummy: float | None = None) -> None: ...
    def ontimer(self, fun: Callable[[], object], t: int = 0) -> None: ...

    @overload
    def bgpic(self, picname: None = None) -> str: ...
    @overload
    def bgpic(self, picname: str) -> None: ...

    @overload
    def screensize(self, canvwidth: None = None, canvheight: None = None, bg: None = None) -> tuple[int, int]: ...
    # Looks like if self.cv is not a ScrolledCanvas, this could return a tuple as well
    @overload
    def screensize(self, canvwidth: int, canvheight: int, bg: _Color | None = None) -> None: ...

    if sys.version_info >= (3, 14):
        def save(self, filename: StrPath, *, overwrite: bool = False) -> None: ...
    onscreenclick = onclick
    resetscreen = reset
    clearscreen = clear
    addshape = register_shape
    def onkeypress(self, fun: Callable[[], object], key: str | None = None) -> None: ...
    onkeyrelease = onkey

class TNavigator:
    START_ORIENTATION: dict[str, Vec2D]
    DEFAULT_MODE: str
    DEFAULT_ANGLEOFFSET: int
    DEFAULT_ANGLEORIENT: int
    def __init__(self, mode: Literal["standard", "logo", "world"] = "standard") -> None: ...
    def reset(self) -> None: ...
    def degrees(self, fullcircle: float = 360.0) -> None: ...
    def radians(self) -> None: ...
    if sys.version_info >= (3, 12):
        def teleport(self, x: float | None = None, y: float | None = None, *, fill_gap: bool = False) -> None: ...

    def forward(self, distance: float) -> None: ...
    def back(self, distance: float) -> None: ...
    def right(self, angle: float) -> None: ...
    def left(self, angle: float) -> None: ...
    def pos(self) -> Vec2D: ...
    def xcor(self) -> float: ...
    def ycor(self) -> float: ...

    @overload
    def goto(self, x: tuple[float, float], y: None = None) -> None: ...
    @overload
    def goto(self, x: float, y: float) -> None: ...

    def home(self) -> None: ...
    def setx(self, x: float) -> None: ...
    def sety(self, y: float) -> None: ...

    @overload
    def distance(self, x: TNavigator | tuple[float, float], y: None = None) -> float: ...
    @overload
    def distance(self, x: float, y: float) -> float: ...

    @overload
    def towards(self, x: TNavigator | tuple[float, float], y: None = None) -> float: ...
    @overload
    def towards(self, x: float, y: float) -> float: ...

    def heading(self) -> float: ...
    def setheading(self, to_angle: float) -> None: ...
    def circle(self, radius: float, extent: float | None = None, steps: int | None = None) -> None: ...
    def speed(self, s: int | None = 0) -> int | None: ...
    fd = forward
    bk = back
    backward = back
    rt = right
    lt = left
    position = pos
    setpos = goto
    setposition = goto
    seth = setheading

class TPen:
    def __init__(self, resizemode: Literal["auto", "user", "noresize"] = "noresize") -> None: ...

    @overload
    def resizemode(self, rmode: None = None) -> str: ...
    @overload
    def resizemode(self, rmode: Literal["auto", "user", "noresize"]) -> None: ...

    @overload
    def pensize(self, width: None = None) -> int: ...
    @overload
    def pensize(self, width: int) -> None: ...

    def penup(self) -> None: ...
    def pendown(self) -> None: ...
    def isdown(self) -> bool: ...

    @overload
    def speed(self, speed: None = None) -> int: ...
    @overload
    def speed(self, speed: _Speed) -> None: ...

    @overload
    def pencolor(self) -> _AnyColor: ...
    @overload
    def pencolor(self, color: _Color) -> None: ...
    @overload
    def pencolor(self, r: float, g: float, b: float) -> None: ...

    @overload
    def fillcolor(self) -> _AnyColor: ...
    @overload
    def fillcolor(self, color: _Color) -> None: ...
    @overload
    def fillcolor(self, r: float, g: float, b: float) -> None: ...

    @overload
    def color(self) -> tuple[_AnyColor, _AnyColor]: ...
    @overload
    def color(self, color: _Color) -> None: ...
    @overload
    def color(self, r: float, g: float, b: float) -> None: ...
    @overload
    def color(self, color1: _Color, color2: _Color) -> None: ...

    if sys.version_info >= (3, 12):
        def teleport(self, x: float | None = None, y: float | None = None, *, fill_gap: bool = False) -> None: ...

    def showturtle(self) -> None: ...
    def hideturtle(self) -> None: ...
    def isvisible(self) -> bool: ...

    # Note: signatures 1 and 2 overlap unsafely when no arguments are provided
    @overload
    def pen(self) -> _PenState: ...
    @overload
    def pen(
        self,
        pen: _PenState | None = None,
        *,
        shown: bool = ...,
        pendown: bool = ...,
        pencolor: _Color = ...,
        fillcolor: _Color = ...,
        pensize: int = ...,
        speed: int = ...,
        resizemode: Literal["auto", "user", "noresize"] = ...,
        stretchfactor: tuple[float, float] = ...,
        outline: int = ...,
        tilt: float = ...,
    ) -> None: ...

    width = pensize
    up = penup
    pu = penup
    pd = pendown
    down = pendown
    st = showturtle
    ht = hideturtle

class RawTurtle(TPen, TNavigator):  # type: ignore[misc]  # Conflicting methods in base classes
    screen: TurtleScreen
    screens: ClassVar[list[TurtleScreen]]
    def __init__(
        self,
        canvas: Canvas | TurtleScreen | None = None,
        shape: str = "classic",
        undobuffersize: int = 1000,
        visible: bool = True,
    ) -> None: ...
    def reset(self) -> None: ...
    def setundobuffer(self, size: int | None) -> None: ...
    def undobufferentries(self) -> int: ...
    def clear(self) -> None: ...
    def clone(self) -> Self: ...

    @overload
    def shape(self, name: None = None) -> str: ...
    @overload
    def shape(self, name: str) -> None: ...

    # Unsafely overlaps when no arguments are provided
    @overload
    def shapesize(self) -> tuple[float, float, float]: ...
    @overload
    def shapesize(
        self, stretch_wid: float | None = None, stretch_len: float | None = None, outline: float | None = None
    ) -> None: ...

    @overload
    def shearfactor(self, shear: None = None) -> float: ...
    @overload
    def shearfactor(self, shear: float) -> None: ...

    # Unsafely overlaps when no arguments are provided
    @overload
    def shapetransform(self) -> tuple[float, float, float, float]: ...
    @overload
    def shapetransform(
        self, t11: float | None = None, t12: float | None = None, t21: float | None = None, t22: float | None = None
    ) -> None: ...

    def get_shapepoly(self) -> _PolygonCoords | None: ...

    if sys.version_info < (3, 13):
        @deprecated("Deprecated since Python 3.1; removed in Python 3.13. Use `tiltangle()` instead.")
        def settiltangle(self, angle: float) -> None: ...

    @overload
    def tiltangle(self, angle: None = None) -> float: ...
    @overload
    def tiltangle(self, angle: float) -> None: ...

    def tilt(self, angle: float) -> None: ...
    # Can return either 'int' or Tuple[int, ...] based on if the stamp is
    # a compound stamp or not. So, as per the "no Union return" policy,
    # we return Any.
    def stamp(self) -> Any: ...
    def clearstamp(self, stampid: int | tuple[int, ...]) -> None: ...
    def clearstamps(self, n: int | None = None) -> None: ...
    def filling(self) -> bool: ...
    if sys.version_info >= (3, 14):
        @contextmanager
        def fill(self) -> Generator[None]: ...

    def begin_fill(self) -> None: ...
    def end_fill(self) -> None: ...

    @overload
    def dot(self, size: int | _Color | None = None) -> None: ...
    @overload
    def dot(self, size: int | None, color: _Color, /) -> None: ...
    @overload
    def dot(self, size: int | None, r: float, g: float, b: float, /) -> None: ...

    def write(
        self, arg: object, move: bool = False, align: str = "left", font: tuple[str, int, str] = ("Arial", 8, "normal")
    ) -> None: ...
    if sys.version_info >= (3, 14):
        @contextmanager
        def poly(self) -> Generator[None]: ...

    def begin_poly(self) -> None: ...
    def end_poly(self) -> None: ...
    def get_poly(self) -> _PolygonCoords | None: ...
    def getscreen(self) -> TurtleScreen: ...
    def getturtle(self) -> Self: ...
    getpen = getturtle
    def onclick(self, fun: Callable[[float, float], object], btn: int = 1, add: bool | None = None) -> None: ...
    def onrelease(self, fun: Callable[[float, float], object], btn: int = 1, add: bool | None = None) -> None: ...
    def ondrag(self, fun: Callable[[float, float], object], btn: int = 1, add: bool | None = None) -> None: ...
    def undo(self) -> None: ...
    turtlesize = shapesize

class _Screen(TurtleScreen):
    def __init__(self) -> None: ...
    # Note int and float are interpreted differently, hence the Union instead of just float
    def setup(
        self,
        width: int | float = 0.5,  # noqa: Y041
        height: int | float = 0.75,  # noqa: Y041
        startx: int | None = None,
        starty: int | None = None,
    ) -> None: ...
    def title(self, titlestring: str) -> None: ...
    def bye(self) -> None: ...
    def exitonclick(self) -> None: ...

class Turtle(RawTurtle):
    def __init__(self, shape: str = "classic", undobuffersize: int = 1000, visible: bool = True) -> None: ...

RawPen = RawTurtle
Pen = Turtle

def write_docstringdict(filename: str = "turtle_docstringdict") -> None: ...

# Functions copied from TurtleScreenBase:

def mainloop() -> None: ...
def textinput(title: str, prompt: str) -> str | None: ...
def numinput(
    title: str, prompt: str, default: float | None = None, minval: float | None = None, maxval: float | None = None
) -> float | None: ...

# Functions copied from TurtleScreen:

def clear() -> None: ...

@overload
def mode(mode: None = None) -> str: ...
@overload
def mode(mode: Literal["standard", "logo", "world"]) -> None: ...

def setworldcoordinates(llx: float, lly: float, urx: float, ury: float) -> None: ...
def register_shape(name: str, shape: _PolygonCoords | Shape | None = None) -> None: ...

@overload
def colormode(cmode: None = None) -> float: ...
@overload
def colormode(cmode: float) -> None: ...

def reset() -> None: ...
def turtles() -> list[Turtle]: ...

@overload
def bgcolor() -> _AnyColor: ...
@overload
def bgcolor(color: _Color) -> None: ...
@overload
def bgcolor(r: float, g: float, b: float) -> None: ...

@overload
def tracer(n: None = None) -> int: ...
@overload
def tracer(n: int, delay: int | None = None) -> None: ...

@overload
def delay(delay: None = None) -> int: ...
@overload
def delay(delay: int) -> None: ...

if sys.version_info >= (3, 14):
    @contextmanager
    def no_animation() -> Generator[None]: ...

def update() -> None: ...
def window_width() -> int: ...
def window_height() -> int: ...
def getcanvas() -> Canvas: ...
def getshapes() -> list[str]: ...
def onclick(fun: Callable[[float, float], object], btn: int = 1, add: bool | None = None) -> None: ...
def onkey(fun: Callable[[], object], key: str) -> None: ...
def listen(xdummy: float | None = None, ydummy: float | None = None) -> None: ...
def ontimer(fun: Callable[[], object], t: int = 0) -> None: ...

@overload
def bgpic(picname: None = None) -> str: ...
@overload
def bgpic(picname: str) -> None: ...

@overload
def screensize(canvwidth: None = None, canvheight: None = None, bg: None = None) -> tuple[int, int]: ...
@overload
def screensize(canvwidth: int, canvheight: int, bg: _Color | None = None) -> None: ...

if sys.version_info >= (3, 14):
    def save(filename: StrPath, *, overwrite: bool = False) -> None: ...

onscreenclick = onclick
resetscreen = reset
clearscreen = clear
addshape = register_shape

def onkeypress(fun: Callable[[], object], key: str | None = None) -> None: ...

onkeyrelease = onkey

# Functions copied from _Screen:

def setup(width: float = 0.5, height: float = 0.75, startx: int | None = None, starty: int | None = None) -> None: ...
def title(titlestring: str) -> None: ...
def bye() -> None: ...
def exitonclick() -> None: ...
def Screen() -> _Screen: ...

# Functions copied from TNavigator:

def degrees(fullcircle: float = 360.0) -> None: ...
def radians() -> None: ...
def forward(distance: float) -> None: ...
def back(distance: float) -> None: ...
def right(angle: float) -> None: ...
def left(angle: float) -> None: ...
def pos() -> Vec2D: ...
def xcor() -> float: ...
def ycor() -> float: ...

@overload
def goto(x: tuple[float, float], y: None = None) -> None: ...
@overload
def goto(x: float, y: float) -> None: ...

def home() -> None: ...
def setx(x: float) -> None: ...
def sety(y: float) -> None: ...

@overload
def distance(x: TNavigator | tuple[float, float], y: None = None) -> float: ...
@overload
def distance(x: float, y: float) -> float: ...

@overload
def towards(x: TNavigator | tuple[float, float], y: None = None) -> float: ...
@overload
def towards(x: float, y: float) -> float: ...

def heading() -> float: ...
def setheading(to_angle: float) -> None: ...
def circle(radius: float, extent: float | None = None, steps: int | None = None) -> None: ...

fd = forward
bk = back
backward = back
rt = right
lt = left
position = pos
setpos = goto
setposition = goto
seth = setheading

# Functions copied from TPen:
@overload
def resizemode(rmode: None = None) -> str: ...
@overload
def resizemode(rmode: Literal["auto", "user", "noresize"]) -> None: ...

@overload
def pensize(width: None = None) -> int: ...
@overload
def pensize(width: int) -> None: ...

def penup() -> None: ...
def pendown() -> None: ...
def isdown() -> bool: ...

@overload
def speed(speed: None = None) -> int: ...
@overload
def speed(speed: _Speed) -> None: ...

@overload
def pencolor() -> _AnyColor: ...
@overload
def pencolor(color: _Color) -> None: ...
@overload
def pencolor(r: float, g: float, b: float) -> None: ...

@overload
def fillcolor() -> _AnyColor: ...
@overload
def fillcolor(color: _Color) -> None: ...
@overload
def fillcolor(r: float, g: float, b: float) -> None: ...

@overload
def color() -> tuple[_AnyColor, _AnyColor]: ...
@overload
def color(color: _Color) -> None: ...
@overload
def color(r: float, g: float, b: float) -> None: ...
@overload
def color(color1: _Color, color2: _Color) -> None: ...

def showturtle() -> None: ...
def hideturtle() -> None: ...
def isvisible() -> bool: ...

# Note: signatures 1 and 2 overlap unsafely when no arguments are provided
@overload
def pen() -> _PenState: ...
@overload
def pen(
    pen: _PenState | None = None,
    *,
    shown: bool = ...,
    pendown: bool = ...,
    pencolor: _Color = ...,
    fillcolor: _Color = ...,
    pensize: int = ...,
    speed: int = ...,
    resizemode: Literal["auto", "user", "noresize"] = ...,
    stretchfactor: tuple[float, float] = ...,
    outline: int = ...,
    tilt: float = ...,
) -> None: ...

width = pensize
up = penup
pu = penup
pd = pendown
down = pendown
st = showturtle
ht = hideturtle

# Functions copied from RawTurtle:

def setundobuffer(size: int | None) -> None: ...
def undobufferentries() -> int: ...

@overload
def shape(name: None = None) -> str: ...
@overload
def shape(name: str) -> None: ...

if sys.version_info >= (3, 12):
    def teleport(x: float | None = None, y: float | None = None, *, fill_gap: bool = False) -> None: ...

# Unsafely overlaps when no arguments are provided
@overload
def shapesize() -> tuple[float, float, float]: ...
@overload
def shapesize(stretch_wid: float | None = None, stretch_len: float | None = None, outline: float | None = None) -> None: ...

@overload
def shearfactor(shear: None = None) -> float: ...
@overload
def shearfactor(shear: float) -> None: ...

# Unsafely overlaps when no arguments are provided
@overload
def shapetransform() -> tuple[float, float, float, float]: ...
@overload
def shapetransform(
    t11: float | None = None, t12: float | None = None, t21: float | None = None, t22: float | None = None
) -> None: ...

def get_shapepoly() -> _PolygonCoords | None: ...

if sys.version_info < (3, 13):
    @deprecated("Deprecated since Python 3.1; removed in Python 3.13. Use `tiltangle()` instead.")
    def settiltangle(angle: float) -> None: ...

@overload
def tiltangle(angle: None = None) -> float: ...
@overload
def tiltangle(angle: float) -> None: ...

def tilt(angle: float) -> None: ...

# Can return either 'int' or Tuple[int, ...] based on if the stamp is
# a compound stamp or not. So, as per the "no Union return" policy,
# we return Any.
def stamp() -> Any: ...
def clearstamp(stampid: int | tuple[int, ...]) -> None: ...
def clearstamps(n: int | None = None) -> None: ...
def filling() -> bool: ...

if sys.version_info >= (3, 14):
    @contextmanager
    def fill() -> Generator[None]: ...

def begin_fill() -> None: ...
def end_fill() -> None: ...

@overload
def dot(size: int | _Color | None = None) -> None: ...
@overload
def dot(size: int | None, color: _Color, /) -> None: ...
@overload
def dot(size: int | None, r: float, g: float, b: float, /) -> None: ...

def write(arg: object, move: bool = False, align: str = "left", font: tuple[str, int, str] = ("Arial", 8, "normal")) -> None: ...

if sys.version_info >= (3, 14):
    @contextmanager
    def poly() -> Generator[None]: ...

def begin_poly() -> None: ...
def end_poly() -> None: ...
def get_poly() -> _PolygonCoords | None: ...
def getscreen() -> TurtleScreen: ...
def getturtle() -> Turtle: ...

getpen = getturtle

def onrelease(fun: Callable[[float, float], object], btn: int = 1, add: bool | None = None) -> None: ...
def ondrag(fun: Callable[[float, float], object], btn: int = 1, add: bool | None = None) -> None: ...
def undo() -> None: ...

turtlesize = shapesize

# Functions copied from RawTurtle with a few tweaks:

def clone() -> Turtle: ...

# Extra functions present only in the global scope:

done = mainloop



from .authorization_server import AuthorizationServer as AuthorizationServer
from .endpoints import RevocationEndpoint as RevocationEndpoint
from .resource_protector import BearerTokenValidator as BearerTokenValidator, ResourceProtector as ResourceProtector
from .signals import (
    client_authenticated as client_authenticated,
    token_authenticated as token_authenticated,
    token_revoked as token_revoked,
)
